#!/bin/bash
# Orbizen Cloud CLI installer script for Linux/macOS

set -e

# Colors
COLOR_RESET='\033[0m'
COLOR_CYAN='\033[0;36m'
COLOR_GREEN='\033[0;32m'
COLOR_RED='\033[0;31m'
COLOR_YELLOW='\033[0;33m'

# Check if Python is installed
if ! command -v python3 &>/dev/null; then
    echo -e "${COLOR_RED}Error: Python 3 is required but not installed.${COLOR_RESET}"
    echo -e "Please install Python 3.8 or later and try again."
    exit 1
fi

# Check Python version
PYTHON_VERSION=$(python3 -c 'import sys; print(f"{sys.version_info.major}.{sys.version_info.minor}")')
if [[ $(echo "$PYTHON_VERSION < 3.8" | bc) -eq 1 ]]; then
    echo -e "${COLOR_RED}Error: Python 3.8 or later is required.${COLOR_RESET}"
    echo -e "Current version: $PYTHON_VERSION"
    echo -e "Please upgrade your Python installation and try again."
    exit 1
fi

# Check if pip is installed
if ! command -v pip3 &>/dev/null; then
    echo -e "${COLOR_YELLOW}Warning: pip3 is not installed. Attempting to install...${COLOR_RESET}"
    if [[ "$OSTYPE" == "darwin"* ]]; then
        # macOS
        python3 -m ensurepip --upgrade
    else
        # Linux
        if command -v apt-get &>/dev/null; then
            sudo apt-get update
            sudo apt-get install -y python3-pip
        elif command -v yum &>/dev/null; then
            sudo yum install -y python3-pip
        elif command -v dnf &>/dev/null; then
            sudo dnf install -y python3-pip
        else
            echo -e "${COLOR_RED}Error: Unable to install pip. Please install pip manually and try again.${COLOR_RESET}"
            exit 1
        fi
    fi
fi

# Install or upgrade Orbizen CLI
echo -e "${COLOR_CYAN}Installing Orbizen Cloud CLI...${COLOR_RESET}"
pip3 install --upgrade orbizen-cli

# Verify installation
if command -v orbizen &>/dev/null; then
    VERSION=$(orbizen -v)
    echo -e "${COLOR_GREEN}Orbizen Cloud CLI $VERSION installed successfully!${COLOR_RESET}"
    echo -e "Run ${COLOR_CYAN}orbizen${COLOR_RESET} to get started."
else
    echo -e "${COLOR_RED}Installation failed.${COLOR_RESET}"
    echo -e "Please try installing manually with: ${COLOR_CYAN}pip3 install orbizen-cli${COLOR_RESET}"
    exit 1
fi 