#!/usr/bin/env python3
import os
import sys
import click
from colorama import init, Fore, Style

# Import command modules
from orbizen.commands import (
    version_cmd,
    auth_cmd,
    init_cmd,
    project_cmd,
    remote_cmd,
    status_cmd,
    media_cmd,
)

from orbizen.utils.config import get_version

# Initialize colorama
init(autoreset=True)

@click.group(invoke_without_command=True)
@click.option('--debug/--no-debug', default=False, help='Enable debug mode')
@click.pass_context
def cli(ctx, debug):
    """Orbizen Cloud CLI - Deploy and manage web applications with ease."""
    ctx.ensure_object(dict)
    ctx.obj['DEBUG'] = debug
    
    # Display help and version if no subcommand is provided
    if ctx.invoked_subcommand is None:
        click.echo(f"{Fore.CYAN}Orbizen Cloud CLI {get_version()}{Style.RESET_ALL}")
        click.echo("Deploy and manage web applications with ease.")
        click.echo()
        click.echo("Common commands:")
        click.echo(f"  {Fore.GREEN}init{Style.RESET_ALL}           Initialize an Orbizen project in the current directory")
        click.echo(f"  {Fore.GREEN}login{Style.RESET_ALL}          Authenticate with Orbizen Cloud")
        click.echo(f"  {Fore.GREEN}remote cloud{Style.RESET_ALL}   Set up remote project on Orbizen Cloud")
        click.echo(f"  {Fore.GREEN}add{Style.RESET_ALL}            Stage files for deployment")
        click.echo(f"  {Fore.GREEN}commit{Style.RESET_ALL}         Create a versioned snapshot")
        click.echo(f"  {Fore.GREEN}remote -u deploy{Style.RESET_ALL} Deploy your project to Orbizen Cloud")
        click.echo()
        click.echo(f"Run {Fore.YELLOW}orbizen COMMAND --help{Style.RESET_ALL} for more information on a command.")
        click.echo()

# Add commands safely
def safe_add_command(command_module, command_name):
    try:
        command = getattr(command_module, command_name)
        cli.add_command(command)
        return True
    except (AttributeError, NameError) as e:
        print(f"Warning: Could not add command {command_name}: {str(e)}")
        return False

# Version command
safe_add_command(version_cmd, 'version')

# Authentication commands
safe_add_command(auth_cmd, 'login')
safe_add_command(auth_cmd, 'register')

# Project initialization
safe_add_command(init_cmd, 'init')

# Project management
safe_add_command(project_cmd, 'add')
safe_add_command(project_cmd, 'commit')
safe_add_command(project_cmd, 'list_project')
safe_add_command(project_cmd, 'select_project')
safe_add_command(project_cmd, 'delete_project')
safe_add_command(project_cmd, 'open_terminal')
safe_add_command(project_cmd, 'close_terminal')
safe_add_command(project_cmd, 'set_domain')
safe_add_command(project_cmd, 'set_php_version')
safe_add_command(project_cmd, 'set_visibility')
try:
    clone_cmd = project_cmd.project_clone
    cli.add_command(clone_cmd, name="clone")
except Exception as e:
    print(f"Warning: Could not add command clone: {str(e)}")

# Remote operations
safe_add_command(remote_cmd, 'remote')
safe_add_command(remote_cmd, 'pull')
# Commented out to avoid conflict with project_cmd.clone
# safe_add_command(remote_cmd, 'clone')
safe_add_command(remote_cmd, 'merge')

# Status command
safe_add_command(status_cmd, 'status')

# Media operations
safe_add_command(media_cmd, 'media')

# Feedback and donations
@cli.command()
def feedback():
    """Submit feedback to Orbizen Cloud team."""
    click.echo("Opening feedback form in your browser...")
    click.launch("https://orbizen.cloud/feedback")

@cli.command()
@click.option('--amount', type=float, help='Donation amount')
def donate(amount):
    """Make a donation to support Orbizen Cloud development."""
    url = "https://orbizen.cloud/donate"
    if amount:
        url += f"?amount={amount}"
    
    click.echo("Opening donation page in your browser...")
    click.launch(url)

@cli.command()
@click.option('-v', '--verbose', is_flag=True, help="Display verbose version information")
def version(verbose):
    """Display the current version of Orbizen CLI."""
    if verbose:
        click.echo(f"{Fore.CYAN}Orbizen Cloud CLI {get_version()}{Style.RESET_ALL}")
        click.echo("© 2023 Orbizen Cloud. All Rights Reserved.")
    else:
        click.echo(get_version())

def main():
    try:
        cli(obj={})
    except Exception as e:
        if '--debug' in sys.argv:
            raise
        else:
            click.echo(f"{Fore.RED}Error: {str(e)}{Style.RESET_ALL}")
            sys.exit(1)

if __name__ == '__main__':
    main() 