"""
Authentication commands for Orbizen CLI.
"""
import os
import json
import click
import webbrowser
import http.server
import socketserver
import threading
import urllib.parse
from datetime import datetime
from colorama import Fore, Style

from orbizen.utils.config import save_global_credentials

# Authentication callback server
class AuthCallbackHandler(http.server.SimpleHTTPRequestHandler):
    def do_GET(self):
        """Handle GET requests to the callback server."""
        # Hidden debug output - Remove or comment this for production
        # print(f"Received callback: {self.path}")
        
        query = urllib.parse.urlparse(self.path).query
        if query:
            params = dict(urllib.parse.parse_qsl(query))
            # Hide sensitive information in logs
            if 'token' in params and len(params['token']) > 20:
                debug_params = params.copy()
                debug_params['token'] = debug_params['token'][:10] + '...' + debug_params['token'][-5:]
                print(f"Authentication callback received with user: {debug_params.get('email', 'unknown')}")
            
            if "token" in params:
                # Save the token and expiration
                credentials = {
                    "token": params["token"],
                    "user_id": params.get("user_id", ""),
                    "email": params.get("email", ""),
                    "created_at": datetime.now().isoformat(),
                    "expires_at": params.get("expires_at", ""),
                    "refresh_token": params.get("refresh_token", "")
                }
                save_global_credentials(credentials)
                
                # Return success page
                self.send_response(200)
                self.send_header("Content-type", "text/html")
                self.end_headers()
                
                success_html = """
                <!DOCTYPE html>
                <html>
                <head>
                    <title>Orbizen Cloud Authentication</title>
                    <style>
                        body {
                            font-family: Arial, sans-serif;
                            text-align: center;
                            margin-top: 50px;
                            background-color: #f5f5f5;
                        }
                        .container {
                            max-width: 500px;
                            margin: 0 auto;
                            padding: 30px;
                            background: white;
                            border-radius: 8px;
                            box-shadow: 0 2px 10px rgba(0,0,0,0.1);
                        }
                        .success {
                            color: green;
                            font-size: 24px;
                            margin-bottom: 20px;
                        }
                        .message {
                            margin-bottom: 30px;
                            line-height: 1.5;
                        }
                        .close-button {
                            display: inline-block;
                            padding: 10px 20px;
                            background-color: #0066cc;
                            color: white;
                            border-radius: 4px;
                            text-decoration: none;
                            cursor: pointer;
                        }
                    </style>
                    <script>
                        window.onload = function() {
                            document.getElementById('close-btn').onclick = function() {
                                window.close();
                                return false;
                            };
                        };
                    </script>
                </head>
                <body>
                    <div class="container">
                        <h1 class="success">Authentication Successful!</h1>
                        <p class="message">You have successfully authenticated with Orbizen Cloud.<br>You can now close this window and return to the terminal.</p>
                        <a id="close-btn" class="close-button">Close this window</a>
                    </div>
                </body>
                </html>
                """
                
                self.wfile.write(success_html.encode())
                
                # Signal that authentication is complete
                self.server.auth_complete = True
                print(f"{Fore.GREEN}Authentication successful!{Style.RESET_ALL}")
                print(f"Logged in as: {params.get('email', 'unknown')}")
                
                # Print expiration info if available
                if params.get("expires_at"):
                    print(f"Token will expire: {params.get('expires_at')}")
                    
                # Print refresh token info if available
                if params.get("refresh_token"):
                    print(f"Refresh token: {'Yes' if params.get('refresh_token') else 'No'}")
            else:
                # Return error page
                self.send_response(400)
                self.send_header("Content-type", "text/html")
                self.end_headers()
                
                error_html = """
                <!DOCTYPE html>
                <html>
                <head>
                    <title>Orbizen Cloud Authentication</title>
                    <style>
                        body {
                            font-family: Arial, sans-serif;
                            text-align: center;
                            margin-top: 50px;
                        }
                        .error {
                            color: red;
                        }
                    </style>
                </head>
                <body>
                    <h1 class="error">Authentication Failed!</h1>
                    <p>Please try again.</p>
                </body>
                </html>
                """
                
                self.wfile.write(error_html.encode())
        else:
            # Return default page
            self.send_response(200)
            self.send_header("Content-type", "text/html")
            self.end_headers()
            
            default_html = """
            <!DOCTYPE html>
            <html>
            <head>
                <title>Orbizen Cloud Authentication</title>
                <style>
                    body {
                        font-family: Arial, sans-serif;
                        text-align: center;
                        margin-top: 50px;
                    }
                </style>
            </head>
            <body>
                <h1>Waiting for authentication...</h1>
                <p>Please complete the authentication process in your browser.</p>
            </body>
            </html>
            """
            
            self.wfile.write(default_html.encode())
    
    def log_message(self, format, *args):
        """Disable logging."""
        pass

def start_auth_server():
    """Start the authentication callback server."""
    # Find an available port
    for port in range(8000, 8100):
        try:
            server = socketserver.TCPServer(("localhost", port), AuthCallbackHandler)
            server.auth_complete = False
            break
        except OSError:
            continue
    else:
        raise Exception("No available ports found")
    
    # Start the server in a separate thread
    thread = threading.Thread(target=server.serve_forever)
    thread.daemon = True
    thread.start()
    
    return server, port

def stop_auth_server(server):
    """Stop the authentication callback server."""
    server.shutdown()
    server.server_close()

@click.command(name="login", help="Log in to Orbizen Cloud")
def login():
    """Log in to Orbizen Cloud using browser-based authentication."""
    click.echo(f"{Fore.CYAN}Logging in to Orbizen Cloud...{Style.RESET_ALL}")
    
    # Start the authentication server
    server, port = start_auth_server()
    
    # Open the browser
    callback_url = f"http://localhost:{port}"
    auth_url = f"https://orbizen.cloud/api/cli.php?callback={urllib.parse.quote(callback_url)}"
    click.echo(f"Opening browser for authentication...")
    webbrowser.open(auth_url)
    
    # Wait for authentication to complete
    click.echo(f"Waiting for authentication to complete... Press Ctrl+C to cancel.")
    try:
        while not server.auth_complete:
            pass
        
        click.echo(f"{Fore.GREEN}Authentication successful!{Style.RESET_ALL}")
        click.echo(f"For troubleshooting deployment issues, use: {Fore.YELLOW}orbizen deploy --debug{Style.RESET_ALL}")
    except KeyboardInterrupt:
        click.echo(f"\n{Fore.RED}Authentication cancelled.{Style.RESET_ALL}")
    finally:
        # Stop the server
        stop_auth_server(server)

@click.command(name="register", help="Register a new Orbizen Cloud account")
def register():
    """Register a new Orbizen Cloud account using browser-based authentication."""
    click.echo(f"{Fore.CYAN}Registering a new Orbizen Cloud account...{Style.RESET_ALL}")
    
    # Open the browser to the registration page
    auth_url = "https://account.orbizen.cloud/register"
    click.echo(f"Opening browser for registration...")
    webbrowser.open(auth_url)
    
    # Display instructions to the user
    click.echo(f"{Fore.GREEN}Browser opened. Please complete your registration and return to the terminal.{Style.RESET_ALL}")
    click.echo(f"Once registered, run '{Fore.YELLOW}orbizen login{Style.RESET_ALL}' and enter your email and password.") 