"""
Project initialization command for Orbizen CLI.
"""
import os
import json
import yaml
import click
from colorama import Fore, Style

from orbizen.utils.config import (
    get_orbizen_dir,
    get_config_path,
    get_staging_path,
    get_commits_path,
    get_remote_path,
    is_orbizen_project,
)

@click.command(name="init", help="Initialize a new Orbizen project")
@click.option('--force', is_flag=True, help="Force initialization even if already initialized")
def init(force):
    """Initialize a new Orbizen project in the current directory."""
    # Check if the project is already initialized
    if is_orbizen_project() and not force:
        click.echo(f"{Fore.YELLOW}Project already initialized. Use --force to reinitialize.{Style.RESET_ALL}")
        return
    
    # Create the .orbizen directory in the HOME directory
    get_orbizen_dir()  # This ensures ~/.orbizen exists
    
    # Create the .orbizen directory in the CURRENT WORKING DIRECTORY
    cwd_orbizen_dir = os.path.join(os.getcwd(), ".orbizen")
    os.makedirs(cwd_orbizen_dir, exist_ok=True)
    
    # Create the config file
    config = {
        "name": os.path.basename(os.getcwd()),
        "created_at": os.path.getctime(os.getcwd()),
    }
    
    config_path = os.path.join(cwd_orbizen_dir, "config.yaml")
    with open(config_path, 'w') as f:
        yaml.dump(config, f, default_flow_style=False)
    
    # Create empty staging file
    staging_path = os.path.join(cwd_orbizen_dir, "staging.json")
    with open(staging_path, 'w') as f:
        json.dump([], f)
    
    # Create empty commits file
    commits_path = os.path.join(cwd_orbizen_dir, "commits.json")
    with open(commits_path, 'w') as f:
        json.dump([], f)
    
    click.echo(f"{Fore.GREEN}Initialized Orbizen project in {os.getcwd()}{Style.RESET_ALL}")
    click.echo(f"Next steps:")
    click.echo(f"  1. {Fore.CYAN}orbizen add .{Style.RESET_ALL} - Stage files for your first commit")
    click.echo(f"  2. {Fore.CYAN}orbizen commit \"Initial commit\"{Style.RESET_ALL} - Create your first commit")
    click.echo(f"  3. {Fore.CYAN}orbizen remote cloud{Style.RESET_ALL} - Create a project on Orbizen Cloud") 