"""
Login command for Orbizen CLI.
"""
import os
import sys
import json
import time
import click
import requests
from colorama import Fore, Style
from pathlib import Path

from orbizen.utils.config import (
    get_orbizen_dir,
    get_auth_path,
    save_auth_token,
    get_auth_token,
)

def save_credentials(user_id, api_token, refresh_token=None, expires_in=86400):
    """Save the credentials to the global config file."""
    # Calculate expiration time
    expires_at = time.time() + expires_in
    
    # Create credentials object
    credentials = {
        "user_id": user_id,
        "token": api_token,
        "refresh_token": refresh_token,
        "expires_at": expires_at
    }
    
    # Get the credentials file path
    home = Path.home()
    orbizen_dir = home / ".orbizen"
    orbizen_dir.mkdir(exist_ok=True)
    creds_file = orbizen_dir / "credentials.json"
    
    # Save the credentials
    with open(creds_file, "w") as f:
        json.dump(credentials, f)
    
    return True

@click.command(name="login", help="Login to Orbizen Cloud")
def login():
    """Login to Orbizen Cloud."""
    # Check if already logged in
    if get_auth_token():
        click.echo(f"{Fore.YELLOW}You are already logged in.{Style.RESET_ALL}")
        if not click.confirm("Do you want to log in again?", default=False):
            return

    # Open browser for registration if needed
    if not click.confirm("Do you have an Orbizen Cloud account?", default=True):
        click.echo(f"{Fore.CYAN}Opening registration page...{Style.RESET_ALL}")
        webbrowser.open("https://account.orbizen.cloud/register")
        click.echo(f"{Fore.YELLOW}Please register and then run 'orbizen login' again.{Style.RESET_ALL}")
        return

    # Get credentials
    click.echo(f"{Fore.CYAN}Please enter your Orbizen Cloud credentials:{Style.RESET_ALL}")
    email = click.prompt("Email")
    password = click.prompt("Password", hide_input=True)

    try:
        click.echo(f"{Fore.CYAN}Authenticating...{Style.RESET_ALL}")
        
        # Make request to login API
        response = requests.post(
            "https://orbizen.cloud/api/auth/login",
            json={"email": email, "password": password},
            headers={"Content-Type": "application/json"}
        )
        
        # Check response
        if response.status_code == 200:
            data = response.json()
            
            # Extract data from response
            if "token" in data and "user_id" in data:
                # Save credentials
                refresh_token = data.get("refresh_token")
                expires_in = data.get("expires_in", 86400)  # Default to 24 hours
                
                save_credentials(
                    data["user_id"],
                    data["token"],
                    refresh_token,
                    expires_in
                )
                
                click.echo(f"{Fore.GREEN}Login successful! You are now logged in to Orbizen Cloud.{Style.RESET_ALL}")
                click.echo(f"{Fore.GREEN}User ID: {data['user_id']}{Style.RESET_ALL}")
                click.echo(f"{Fore.GREEN}Token will expire in: {expires_in//3600} hours{Style.RESET_ALL}")
                
                # Print debug info about the token format
                print(f"{Fore.BLUE}Debug - Token length: {len(data['token'])}{Style.RESET_ALL}")
                if len(data["token"]) < 20:
                    print(f"{Fore.YELLOW}Warning: Token seems unusually short.{Style.RESET_ALL}")
                
                return True
            else:
                click.echo(f"{Fore.RED}Login failed: Invalid response from server.{Style.RESET_ALL}")
                return False
        else:
            error_msg = response.json().get("message", "Unknown error") if response.headers.get("content-type") == "application/json" else response.text
            click.echo(f"{Fore.RED}Login failed: {error_msg}{Style.RESET_ALL}")
            return False
    except requests.exceptions.RequestException as e:
        click.echo(f"{Fore.RED}Error connecting to Orbizen Cloud: {str(e)}{Style.RESET_ALL}")
    except Exception as e:
        click.echo(f"{Fore.RED}An error occurred: {str(e)}{Style.RESET_ALL}") 