"""
Media commands for Orbizen CLI.
"""
import os
import click
from colorama import Fore, Style
from orbizen.utils.media import compress_video, batch_compress_videos, check_ffmpeg_installed
from orbizen.utils.config import is_orbizen_project

@click.group(name="media", help="Media file operations")
def media():
    """Media file operations for Orbizen Cloud."""
    pass

@media.command(name="compress-video", help="Compress a video file")
@click.argument('input_path', type=click.Path(exists=True))
@click.option('-o', '--output', type=click.Path(), help="Output path for compressed video")
@click.option('-q', '--quality', type=click.Choice(['low', 'medium', 'high']), default='medium', 
              help="Compression quality (default: medium)")
@click.option('-s', '--max-size', type=int, default=50, 
              help="Target maximum size in MB (default: 50)")
def compress_video_cmd(input_path, output, quality, max_size):
    """Compress a video file to reduce its size while maintaining reasonable quality."""
    if not check_ffmpeg_installed():
        click.echo(f"{Fore.RED}FFmpeg is not installed. Please install FFmpeg to use video compression.{Style.RESET_ALL}")
        click.echo(f"{Fore.YELLOW}Visit https://ffmpeg.org/download.html for installation instructions.{Style.RESET_ALL}")
        return
    
    result = compress_video(input_path, output, quality, max_size)
    
    if result:
        click.echo(f"{Fore.GREEN}Video compressed successfully: {result}{Style.RESET_ALL}")
    else:
        click.echo(f"{Fore.RED}Failed to compress video.{Style.RESET_ALL}")

@media.command(name="compress-all", help="Compress all videos in a directory")
@click.argument('directory', type=click.Path(exists=True, file_okay=False, dir_okay=True))
@click.option('-q', '--quality', type=click.Choice(['low', 'medium', 'high']), default='medium', 
              help="Compression quality (default: medium)")
@click.option('-s', '--max-size', type=int, default=50, 
              help="Target maximum size in MB (default: 50)")
@click.option('-e', '--extensions', multiple=True, default=['.mp4', '.mov', '.avi', '.mkv', '.webm'],
              help="Video file extensions to process (can be specified multiple times)")
@click.option('--replace', is_flag=True, help="Replace original files with compressed versions")
def batch_compress_cmd(directory, quality, max_size, extensions, replace):
    """Compress all videos in a directory to reduce their size."""
    if not check_ffmpeg_installed():
        click.echo(f"{Fore.RED}FFmpeg is not installed. Please install FFmpeg to use video compression.{Style.RESET_ALL}")
        click.echo(f"{Fore.YELLOW}Visit https://ffmpeg.org/download.html for installation instructions.{Style.RESET_ALL}")
        return
    
    click.echo(f"{Fore.BLUE}Scanning directory: {directory}{Style.RESET_ALL}")
    click.echo(f"{Fore.BLUE}Quality: {quality}, Max size: {max_size}MB{Style.RESET_ALL}")
    
    compressed_files = batch_compress_videos(directory, quality, max_size, extensions)
    
    if not compressed_files:
        click.echo(f"{Fore.YELLOW}No videos found or compressed.{Style.RESET_ALL}")
        return
    
    click.echo(f"{Fore.GREEN}Compressed {len(compressed_files)} videos.{Style.RESET_ALL}")
    
    if replace:
        click.echo(f"{Fore.BLUE}Replacing original files with compressed versions...{Style.RESET_ALL}")
        for original_path, compressed_path in compressed_files.items():
            if original_path != compressed_path:  # Skip if compression didn't produce a smaller file
                os.replace(compressed_path, original_path)
                click.echo(f"{Fore.GREEN}Replaced: {original_path}{Style.RESET_ALL}")
        
        click.echo(f"{Fore.GREEN}All original files replaced with compressed versions.{Style.RESET_ALL}")

@media.command(name="prepare-videos", help="Prepare videos for deployment")
@click.option('-q', '--quality', type=click.Choice(['low', 'medium', 'high']), default='medium', 
              help="Compression quality (default: medium)")
@click.option('-s', '--max-size', type=int, default=50, 
              help="Target maximum size in MB (default: 50)")
@click.option('-e', '--extensions', multiple=True, default=['.mp4', '.mov', '.avi', '.mkv', '.webm'],
              help="Video file extensions to process (can be specified multiple times)")
def prepare_videos_cmd(quality, max_size, extensions):
    """Prepare videos in the current project for deployment by compressing them."""
    if not is_orbizen_project():
        click.echo(f"{Fore.RED}Not an Orbizen project. Run 'orbizen init' first.{Style.RESET_ALL}")
        return
    
    if not check_ffmpeg_installed():
        click.echo(f"{Fore.RED}FFmpeg is not installed. Please install FFmpeg to use video compression.{Style.RESET_ALL}")
        click.echo(f"{Fore.YELLOW}Visit https://ffmpeg.org/download.html for installation instructions.{Style.RESET_ALL}")
        return
    
    click.echo(f"{Fore.BLUE}Preparing videos for deployment...{Style.RESET_ALL}")
    
    # Get the current directory (project root)
    project_dir = os.getcwd()
    
    # Create a directory for compressed videos if it doesn't exist
    compressed_dir = os.path.join(project_dir, "compressed_videos")
    os.makedirs(compressed_dir, exist_ok=True)
    
    # Find all video files in the project
    video_files = []
    for root, _, files in os.walk(project_dir):
        # Skip the compressed_videos directory
        if os.path.relpath(root, project_dir).startswith("compressed_videos"):
            continue
            
        # Skip the .orbizen directory
        if os.path.relpath(root, project_dir).startswith(".orbizen"):
            continue
            
        for file in files:
            if any(file.lower().endswith(ext) for ext in extensions):
                video_files.append(os.path.join(root, file))
    
    if not video_files:
        click.echo(f"{Fore.YELLOW}No video files found in the project.{Style.RESET_ALL}")
        return
    
    click.echo(f"{Fore.BLUE}Found {len(video_files)} video files.{Style.RESET_ALL}")
    
    # Compress each video file
    compressed_count = 0
    for video_file in video_files:
        # Skip if file is already under the size limit
        if os.path.getsize(video_file) <= max_size * 1024 * 1024:
            rel_path = os.path.relpath(video_file, project_dir)
            click.echo(f"{Fore.BLUE}Skipping {rel_path} - already under {max_size}MB limit{Style.RESET_ALL}")
            continue
            
        # Create output path in the compressed_videos directory
        rel_path = os.path.relpath(video_file, project_dir)
        output_path = os.path.join(compressed_dir, rel_path)
        
        # Create directory structure if needed
        os.makedirs(os.path.dirname(output_path), exist_ok=True)
        
        # Compress the video
        result = compress_video(video_file, output_path, quality, max_size)
        
        if result:
            compressed_count += 1
            
            # Calculate compression ratio
            original_size = os.path.getsize(video_file)
            compressed_size = os.path.getsize(result)
            compression_ratio = (1 - (compressed_size / original_size)) * 100
            
            click.echo(f"{Fore.GREEN}Compressed: {rel_path} - Reduced by {compression_ratio:.2f}%{Style.RESET_ALL}")
    
    if compressed_count > 0:
        click.echo(f"{Fore.GREEN}Successfully compressed {compressed_count} videos.{Style.RESET_ALL}")
        click.echo(f"{Fore.BLUE}Compressed videos are available in the 'compressed_videos' directory.{Style.RESET_ALL}")
        click.echo(f"{Fore.BLUE}You can use these files in your project for deployment.{Style.RESET_ALL}")
    else:
        click.echo(f"{Fore.YELLOW}No videos were compressed. All videos may already be under the size limit.{Style.RESET_ALL}") 