"""
Project management commands for Orbizen CLI.
"""
import os
import sys
import click
import getpass
import subprocess
import json
from tabulate import tabulate
from colorama import Fore, Style

from orbizen.utils.config import is_orbizen_project, get_orbizen_dir
from orbizen.utils.project import (
    stage_file,
    stage_directory,
    get_staged_files,
    create_commit,
    get_commits,
    get_projects,
    get_remote_config,
)
from orbizen.utils.api import make_api_request, setup_custom_domain

@click.command(name="add", help="Stage files for deployment")
@click.argument('path', required=True)
def add(path):
    """Stage files for deployment."""
    # Check if the project is initialized
    if not is_orbizen_project():
        click.echo(f"{Fore.RED}Not an Orbizen project. Run 'orbizen init' first.{Style.RESET_ALL}")
        return
    
    # Stage files
    if os.path.isdir(path):
        # Stage directory
        count = stage_directory(path)
        click.echo(f"{Fore.GREEN}Staged all files in {path}{Style.RESET_ALL}")
    elif os.path.isfile(path):
        # Stage file
        success = stage_file(path)
        if success:
            click.echo(f"{Fore.GREEN}Staged {path}{Style.RESET_ALL}")
        else:
            click.echo(f"{Fore.RED}Failed to stage {path}{Style.RESET_ALL}")
    else:
        # Invalid path
        click.echo(f"{Fore.RED}Invalid path: {path}{Style.RESET_ALL}")

@click.command(name="commit", help="Create a versioned snapshot")
@click.argument('message', required=True)
def commit(message):
    """Create a versioned snapshot."""
    # Check if the project is initialized
    if not is_orbizen_project():
        click.echo(f"{Fore.RED}Not an Orbizen project. Run 'orbizen init' first.{Style.RESET_ALL}")
        return
    
    # Check if there are staged files
    staged_files = get_staged_files()
    if not staged_files:
        click.echo(f"{Fore.RED}No staged files. Run 'orbizen add <file>' first.{Style.RESET_ALL}")
        return
    
    # Create the commit
    success = create_commit(message)
    if success:
        click.echo(f"{Fore.GREEN}Created commit: {message}{Style.RESET_ALL}")
        click.echo(f"Committed {len(staged_files)} files")
    else:
        click.echo(f"{Fore.RED}Failed to create commit{Style.RESET_ALL}")

@click.command(name="list-project", help="List all projects")
@click.option('-v', '--verbose', is_flag=True, help="Display verbose project information")
def list_project(verbose):
    """List all projects."""
    # Get the projects
    response = get_projects()
    
    if not response or response.get('status') != 'success' or not response.get('data'):
        click.echo(f"{Fore.YELLOW}No projects found.{Style.RESET_ALL}")
        return
    
    projects = response.get('data', [])
    
    if not projects:
        click.echo(f"{Fore.YELLOW}No projects found.{Style.RESET_ALL}")
        return
    
    # Format the projects
    headers = ["ID", "Name", "Domain", "Type", "Visibility"]
    rows = []
    
    for project in projects:
        row = [
            project.get("project_id", "N/A"),
            project.get("name", "N/A"),
            project.get("domain", "N/A"),
            project.get("type", "N/A"),
            project.get("visibility", "private")
        ]
        
        rows.append(row)
    
    # Display the projects
    click.echo(tabulate(rows, headers=headers, tablefmt="grid"))

@click.command(name="select", help="Select a project to work with")
@click.option('-p', '--project', required=True, help="The project name or ID to select")
def select_project(project):
    """Select a project to work with."""
    # Get the projects
    response = get_projects()
    
    if not response or response.get('status') != 'success' or not response.get('data'):
        click.echo(f"{Fore.YELLOW}No projects found.{Style.RESET_ALL}")
        return
    
    projects = response.get('data', [])
    
    if not projects:
        click.echo(f"{Fore.YELLOW}No projects found.{Style.RESET_ALL}")
        return
    
    # Find the project by name or ID
    selected_project = None
    for p in projects:
        if (p.get("name", "").lower() == project.lower() or 
            str(p.get("id", "")) == project or 
            p.get("project_id", "") == project):
            selected_project = p
            break
    
    if not selected_project:
        click.echo(f"{Fore.RED}Project not found: {project}{Style.RESET_ALL}")
        return
    
    # Display project details
    click.echo(f"{Fore.GREEN}Project: {selected_project.get('name', 'N/A')}{Style.RESET_ALL}")
    click.echo(f"ID: {selected_project.get('project_id', 'N/A')}")
    click.echo(f"Domain: {selected_project.get('domain', 'N/A')}")
    click.echo(f"Type: {selected_project.get('type', 'N/A')}")
    click.echo(f"PHP Version: {selected_project.get('php_version', 'N/A')}")
    click.echo(f"Created: {selected_project.get('created_at', 'N/A')}")
    click.echo(f"Visibility: {selected_project.get('visibility', 'private')}")
    
    # Check for database
    if "database" in selected_project:
        click.echo(f"\n{Fore.CYAN}Database Information{Style.RESET_ALL}")
        click.echo(f"Database: {selected_project['database'].get('name', 'N/A')}")
        click.echo(f"Username: {selected_project['database'].get('user', 'N/A')}")
        click.echo(f"Password: {selected_project['database'].get('password', 'N/A')}")
    
    # Set as current project
    with open(os.path.join(get_orbizen_dir(), "current_project.json"), "w") as f:
        json.dump(selected_project, f)
    
    click.echo(f"\n{Fore.GREEN}Project selected successfully.{Style.RESET_ALL}")

@click.command(name="open", help="Open a terminal on the server for the selected project")
@click.option('-t', '--terminal', is_flag=True, default=True, help="Open terminal on server")
def open_terminal(terminal):
    """Open a terminal on the server for the selected project."""
    # Check if a project is selected
    current_project_path = os.path.join(get_orbizen_dir(), "current_project.json")
    if not os.path.exists(current_project_path):
        click.echo(f"{Fore.RED}No project selected. Use 'orbizen select -p <project>' first.{Style.RESET_ALL}")
        return
    
    # Get the selected project
    with open(current_project_path, "r") as f:
        project = json.load(f)
    
    # Open SSH terminal for the project
    try:
        ssh_command = f"ssh orbizen@orbizen.cloud -t \"cd /home/orbizen/orbizen.cloud/cli/project/{project['project_id']} && bash\""
        click.echo(f"{Fore.CYAN}Opening terminal for {project['name']}...{Style.RESET_ALL}")
        click.echo(f"Project directory: /home/orbizen/orbizen.cloud/cli/project/{project['project_id']}")
        click.echo(f"Type 'exit' to close the terminal.")
        subprocess.call(ssh_command, shell=True)
    except Exception as e:
        click.echo(f"{Fore.RED}Error opening terminal: {str(e)}{Style.RESET_ALL}")

@click.command(name="close", help="Close the server terminal")
@click.option('-t', '--terminal', is_flag=True, default=True, help="Close terminal on server")
def close_terminal(terminal):
    """Close the server terminal."""
    click.echo(f"{Fore.CYAN}To close the server terminal, type 'exit' in the terminal.{Style.RESET_ALL}")

@click.command(name="clone", help="Clone a project from a domain")
@click.argument('domain', required=True)
@click.option('-e', '--enable', is_flag=True, help="Enable cloning for your project")
@click.option('-d', '--disable', is_flag=True, help="Disable cloning for your project")
def project_clone(domain, enable, disable):
    """Clone a project or manage cloning settings."""
    # Check if we're managing cloning settings for the current project
    if enable or disable:
        # Check if a project is selected
        current_project_path = os.path.join(get_orbizen_dir(), "current_project.json")
        if not os.path.exists(current_project_path):
            click.echo(f"{Fore.RED}No project selected. Use 'orbizen select -p <project>' first.{Style.RESET_ALL}")
            return
        
        # Get the selected project
        with open(current_project_path, "r") as f:
            project = json.load(f)
        
        project_id = project.get('project_id')
        
        if not project_id:
            click.echo(f"{Fore.RED}Invalid project data. Please select a valid project.{Style.RESET_ALL}")
            return
        
        # Update the cloning setting
        try:
            data = {
                "visibility": "public" if enable else "private"
            }
            response = make_api_request(f"projects/{project_id}", method="PUT", data=data)
            
            if response and response.get('status') == 'success':
                if enable:
                    click.echo(f"{Fore.GREEN}Project cloning enabled. Others can now clone your project.{Style.RESET_ALL}")
                    click.echo(f"Note: Your project visibility is now 'public', which allows cloning.")
                else:
                    click.echo(f"{Fore.GREEN}Project cloning disabled. Your project is no longer available for cloning.{Style.RESET_ALL}")
                    click.echo(f"Note: Your project visibility is now 'private', which prevents cloning.")
                
                # Update the local project data
                project['visibility'] = "public" if enable else "private"
                with open(current_project_path, "w") as f:
                    json.dump(project, f)
            else:
                error_msg = response.get('message', 'Unknown error') if response else 'No response'
                click.echo(f"{Fore.RED}Error updating project clone settings: {error_msg}{Style.RESET_ALL}")
        except Exception as e:
            click.echo(f"{Fore.RED}Error updating project clone settings: {str(e)}{Style.RESET_ALL}")
        
        return
    
    # If we're here, we're cloning a project
    try:
        # Check if the domain is valid
        if not domain or not ("." in domain):
            click.echo(f"{Fore.RED}Invalid domain format. Please provide a valid domain.{Style.RESET_ALL}")
            return
        
        click.echo(f"{Fore.CYAN}Cloning project: {domain}...{Style.RESET_ALL}")
        
        # Get all projects and find the one with matching domain
        response = make_api_request("projects")
        
        if not response or response.get('status') != 'success' or not response.get('data'):
            click.echo(f"{Fore.RED}Failed to retrieve projects.{Style.RESET_ALL}")
            return
        
        projects = response.get('data', [])
        project_data = None
        
        for project in projects:
            if project.get('domain') == domain:
                project_data = project
                break
        
        if not project_data:
            click.echo(f"{Fore.RED}Project with domain {domain} not found or not available for cloning.{Style.RESET_ALL}")
            return
        
        # Check if the project is cloneable (public visibility)
        if project_data.get('visibility') != 'public':
            click.echo(f"{Fore.RED}This project is not available for cloning. The owner has not enabled cloning.{Style.RESET_ALL}")
            return
        
        project_id = project_data.get('project_id')
        
        # Create directory for the cloned project
        clone_dir = os.path.join(os.getcwd(), f"cloned_{project_id}")
        os.makedirs(clone_dir, exist_ok=True)
        
        # Download the project files - we'll use a direct download approach
        # Since we don't have a dedicated clone API endpoint, we'll simulate it
        
        # First, create a simple index.html file to show the clone was successful
        with open(os.path.join(clone_dir, "index.html"), "w") as f:
            f.write(f"""<!DOCTYPE html>
<html>
<head>
    <title>Cloned Project: {project_data.get('name')}</title>
    <style>
        body {{ font-family: Arial, sans-serif; margin: 40px; line-height: 1.6; }}
        h1 {{ color: #4CAF50; }}
        .info {{ background: #f5f5f5; padding: 15px; border-radius: 5px; }}
    </style>
</head>
<body>
    <h1>Successfully Cloned Project</h1>
    <div class="info">
        <p><strong>Project Name:</strong> {project_data.get('name')}</p>
        <p><strong>Domain:</strong> {project_data.get('domain')}</p>
        <p><strong>Project ID:</strong> {project_id}</p>
        <p><strong>Type:</strong> {project_data.get('type')}</p>
        <p><strong>PHP Version:</strong> {project_data.get('php_version')}</p>
        <p><strong>Created At:</strong> {project_data.get('created_at')}</p>
        <p><strong>Visibility:</strong> {project_data.get('visibility', 'private')}</p>
    </div>
    <p>This is a placeholder for the cloned project content. In a real implementation, 
    the actual project files would be downloaded from the server.</p>
</body>
</html>""")
        
        # Create a README file with instructions
        with open(os.path.join(clone_dir, "README.md"), "w") as f:
            f.write(f"""# Cloned Project: {project_data.get('name')}

This is a cloned project from {domain}.

## Project Information
- **Name:** {project_data.get('name')}
- **Domain:** {project_data.get('domain')}
- **Project ID:** {project_id}
- **Type:** {project_data.get('type')}
- **PHP Version:** {project_data.get('php_version')}
- **Visibility:** {project_data.get('visibility', 'private')}

## Getting Started
1. Review the files in this directory
2. Make any necessary changes
3. Deploy to your own Orbizen Cloud account

## Note
This is a placeholder implementation. In a complete implementation, the actual
project files would be downloaded from the server.
""")
        
        click.echo(f"{Fore.GREEN}Project cloned successfully to {clone_dir}{Style.RESET_ALL}")
        click.echo(f"{Fore.YELLOW}Note: This is a placeholder implementation. The actual files would be downloaded in a complete implementation.{Style.RESET_ALL}")
    except Exception as e:
        click.echo(f"{Fore.RED}Error cloning project: {str(e)}{Style.RESET_ALL}")

@click.command(name="delete", help="Delete a project")
@click.argument('project_id', required=False)
def delete_project(project_id):
    """Delete a project."""
    # Check if a project is selected if no project_id is provided
    if not project_id:
        current_project_path = os.path.join(get_orbizen_dir(), "current_project.json")
        if not os.path.exists(current_project_path):
            click.echo(f"{Fore.RED}No project selected or specified. Use 'orbizen select -p <project>' first or provide a project ID.{Style.RESET_ALL}")
            return
        
        # Get the selected project
        with open(current_project_path, "r") as f:
            project = json.load(f)
        
        project_id = project.get('project_id')
        
        if not project_id:
            click.echo(f"{Fore.RED}Invalid project data. Please select a valid project.{Style.RESET_ALL}")
            return
    
    # Confirm deletion
    if not click.confirm(f"{Fore.RED}Are you sure you want to delete project {project_id}? This cannot be undone.{Style.RESET_ALL}"):
        click.echo(f"{Fore.YELLOW}Deletion cancelled.{Style.RESET_ALL}")
        return
    
    # Delete the project
    try:
        click.echo(f"{Fore.CYAN}Deleting project {project_id}...{Style.RESET_ALL}")
        
        # Get project info before deletion
        try:
            project_info = make_api_request(f"projects/{project_id}", method="GET")
            if project_info and project_info.get('status') == 'success' and project_info.get('data'):
                domain = project_info['data'].get('domain')
                if domain:
                    click.echo(f"Removing domain: {domain}")
        except Exception:
            pass
        
        # Perform the deletion
        response = make_api_request(f"projects/{project_id}", method="DELETE")
        
        if response and response.get('status') == 'success':
            click.echo(f"{Fore.GREEN}Project deleted successfully.{Style.RESET_ALL}")
            click.echo(f"{Fore.GREEN}All associated domains, subdomains, and files have been removed.{Style.RESET_ALL}")
            
            # Remove current_project.json if this was the current project
            current_project_path = os.path.join(get_orbizen_dir(), "current_project.json")
            if os.path.exists(current_project_path):
                with open(current_project_path, "r") as f:
                    current_project = json.load(f)
                
                if current_project.get('project_id') == project_id:
                    os.remove(current_project_path)
                    click.echo(f"{Fore.YELLOW}Removed local project reference.{Style.RESET_ALL}")
        else:
            error_msg = response.get('message', 'Unknown error') if response else 'No response'
            click.echo(f"{Fore.RED}Error deleting project: {error_msg}{Style.RESET_ALL}")
    except Exception as e:
        click.echo(f"{Fore.RED}Error deleting project: {str(e)}{Style.RESET_ALL}")

@click.command(name="set-domain", help="Set a custom domain for the selected project")
@click.argument('domain', required=True)
def set_domain(domain):
    """Set a custom domain for the selected project."""
    # Check if a project is selected
    current_project_path = os.path.join(get_orbizen_dir(), "current_project.json")
    if not os.path.exists(current_project_path):
        click.echo(f"{Fore.RED}No project selected. Use 'orbizen select -p <project>' first.{Style.RESET_ALL}")
        return
    
    # Get the selected project
    with open(current_project_path, "r") as f:
        project = json.load(f)
    
    project_id = project.get('project_id')
    
    if not project_id:
        click.echo(f"{Fore.RED}Invalid project data. Please select a valid project.{Style.RESET_ALL}")
        return
    
    # Validate domain format
    if not domain or not ("." in domain):
        click.echo(f"{Fore.RED}Invalid domain format. Please provide a valid domain (e.g. example.com).{Style.RESET_ALL}")
        return
    
    # Confirm domain setup
    click.echo(f"{Fore.YELLOW}You are about to set {domain} as the custom domain for project {project.get('name')}.{Style.RESET_ALL}")
    click.echo(f"{Fore.YELLOW}To make this work, you'll need to add an A record in your domain's DNS settings:{Style.RESET_ALL}")
    click.echo(f"{Fore.CYAN}Domain: {domain}{Style.RESET_ALL}")
    click.echo(f"{Fore.CYAN}Type: A{Style.RESET_ALL}")
    click.echo(f"{Fore.CYAN}Value: 144.76.203.197{Style.RESET_ALL}")
    
    if not click.confirm(f"{Fore.YELLOW}Do you want to continue?{Style.RESET_ALL}"):
        click.echo(f"{Fore.YELLOW}Domain setup cancelled.{Style.RESET_ALL}")
        return
    
    # Set up the custom domain
    try:
        click.echo(f"{Fore.CYAN}Setting up custom domain {domain} for project {project_id}...{Style.RESET_ALL}")
        
        # First, set up the domain on the server
        domain_result = setup_custom_domain(domain, project_id)
        
        if domain_result and domain_result.get('status') == 'success':
            # Get visibility and allowed IPs from the project
            visibility = project.get('visibility', 'private')
            allowed_ips = project.get('allowed_ips', [])
            
            # Update the project in the API
            data = {
                "domain": domain,
                "visibility": visibility,
                "allowed_ips": allowed_ips
            }
            response = make_api_request(f"projects/{project_id}", method="PUT", data=data)
            
            if response and response.get('status') == 'success':
                click.echo(f"{Fore.GREEN}Custom domain {domain} set up successfully.{Style.RESET_ALL}")
                click.echo(f"{Fore.GREEN}Your project is now available at: https://{domain}{Style.RESET_ALL}")
                click.echo(f"{Fore.YELLOW}Note: DNS changes may take up to 24-48 hours to propagate globally.{Style.RESET_ALL}")
                
                # Update the local project data
                project['domain'] = domain
                with open(current_project_path, "w") as f:
                    json.dump(project, f)
                
                # Update .htaccess file to enforce visibility restrictions
                from orbizen.utils.api import update_htaccess_for_visibility
                update_htaccess_for_visibility(project_id, visibility, allowed_ips)
            else:
                error_msg = response.get('message', 'Unknown error') if response else 'No response'
                click.echo(f"{Fore.RED}Error updating project domain in database: {error_msg}{Style.RESET_ALL}")
                click.echo(f"{Fore.YELLOW}Domain setup on server was successful, but database update failed.{Style.RESET_ALL}")
        else:
            error_msg = domain_result.get('message', 'Unknown error') if domain_result else 'No response'
            click.echo(f"{Fore.RED}Error setting up custom domain: {error_msg}{Style.RESET_ALL}")
    except Exception as e:
        click.echo(f"{Fore.RED}Error setting up custom domain: {str(e)}{Style.RESET_ALL}")

@click.command(name="set-php", help="Change the PHP version of the selected project")
@click.argument('version', required=True, type=click.Choice(['7.4', '8.0', '8.1', '8.2', '8.3']))
def set_php_version(version):
    """Change the PHP version of the selected project."""
    # Check if a project is selected
    current_project_path = os.path.join(get_orbizen_dir(), "current_project.json")
    if not os.path.exists(current_project_path):
        click.echo(f"{Fore.RED}No project selected. Use 'orbizen select -p <project>' first.{Style.RESET_ALL}")
        return
    
    # Get the selected project
    with open(current_project_path, "r") as f:
        project = json.load(f)
    
    project_id = project.get('project_id')
    
    if not project_id:
        click.echo(f"{Fore.RED}Invalid project data. Please select a valid project.{Style.RESET_ALL}")
        return
    
    # Check if the version is different from the current one
    if project.get('php_version') == version:
        click.echo(f"{Fore.YELLOW}Project {project.get('name')} is already using PHP {version}.{Style.RESET_ALL}")
        return
    
    # Confirm PHP version change
    if not click.confirm(f"{Fore.YELLOW}Change PHP version for project {project.get('name')} from {project.get('php_version')} to {version}?{Style.RESET_ALL}"):
        click.echo(f"{Fore.YELLOW}PHP version change cancelled.{Style.RESET_ALL}")
        return
    
    # Update the PHP version
    try:
        click.echo(f"{Fore.CYAN}Updating PHP version to {version} for project {project_id}...{Style.RESET_ALL}")
        
        # Get visibility and allowed IPs from the project
        visibility = project.get('visibility', 'private')
        allowed_ips = project.get('allowed_ips', [])
        
        # Update the project in the API
        data = {
            "php_version": version,
            "visibility": visibility,
            "allowed_ips": allowed_ips
        }
        response = make_api_request(f"projects/{project_id}", method="PUT", data=data)
        
        if response and response.get('status') == 'success':
            click.echo(f"{Fore.GREEN}PHP version updated successfully to {version}.{Style.RESET_ALL}")
            click.echo(f"{Fore.YELLOW}Note: You may need to redeploy your project for changes to take effect.{Style.RESET_ALL}")
            
            # Update the local project data
            project['php_version'] = version
            with open(current_project_path, "w") as f:
                json.dump(project, f)
        else:
            error_msg = response.get('message', 'Unknown error') if response else 'No response'
            click.echo(f"{Fore.RED}Error updating PHP version: {error_msg}{Style.RESET_ALL}")
    except Exception as e:
        click.echo(f"{Fore.RED}Error updating PHP version: {str(e)}{Style.RESET_ALL}")

@click.command(name="set-visibility", help="Change the visibility of the selected project")
@click.argument('visibility', required=True, type=click.Choice(['public', 'private', 'group']))
@click.option('--ips', help="Comma-separated list of IP addresses for 'group' visibility")
def set_visibility(visibility, ips):
    """Change the visibility of the selected project."""
    # Check if a project is selected
    current_project_path = os.path.join(get_orbizen_dir(), "current_project.json")
    if not os.path.exists(current_project_path):
        click.echo(f"{Fore.RED}No project selected. Use 'orbizen select -p <project>' first.{Style.RESET_ALL}")
        return
    
    # Get the selected project
    with open(current_project_path, "r") as f:
        project = json.load(f)
    
    project_id = project.get('project_id')
    
    if not project_id:
        click.echo(f"{Fore.RED}Invalid project data. Please select a valid project.{Style.RESET_ALL}")
        return
    
    # Check if the visibility is different from the current one
    if project.get('visibility') == visibility:
        click.echo(f"{Fore.YELLOW}Project {project.get('name')} is already {visibility}.{Style.RESET_ALL}")
        return
    
    # Get IP addresses based on visibility type
    allowed_ips = []
    if visibility == 'group':
        if ips:
            allowed_ips = [ip.strip() for ip in ips.split(',')]
        else:
            click.echo(f"Enter IP addresses that can access the website (comma-separated):")
            ip_input = click.prompt("IP Addresses")
            allowed_ips = [ip.strip() for ip in ip_input.split(',')]
    elif visibility == 'private':
        # Get the current IP address automatically
        try:
            import socket
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            # Doesn't need to be reachable, just to determine the interface
            s.connect(('8.8.8.8', 80))
            current_ip = s.getsockname()[0]
            s.close()
            allowed_ips = [current_ip]
            click.echo(f"Automatically added your current IP address: {current_ip}")
        except Exception as e:
            click.echo(f"{Fore.YELLOW}Could not automatically detect your IP address: {str(e)}{Style.RESET_ALL}")
            click.echo(f"Enter your IP address manually:")
            ip_address = click.prompt("IP Address")
            allowed_ips = [ip_address.strip()]
    
    # Confirm visibility change
    message = ""
    if visibility == "public":
        message = "This will make your project publicly accessible to anyone."
    elif visibility == "private":
        message = f"This will restrict access to only your IP address ({', '.join(allowed_ips)})."
    elif visibility == "group":
        message = f"This will restrict access to specific IP addresses ({', '.join(allowed_ips)})."
    
    if not click.confirm(f"{Fore.YELLOW}Change visibility for project {project.get('name')} to {visibility}? {message}{Style.RESET_ALL}"):
        click.echo(f"{Fore.YELLOW}Visibility change cancelled.{Style.RESET_ALL}")
        return
    
    # Update the visibility
    try:
        click.echo(f"{Fore.CYAN}Updating visibility to {visibility} for project {project_id}...{Style.RESET_ALL}")
        
        # Update the project in the API
        data = {
            "visibility": visibility,
            "allowed_ips": allowed_ips
        }
        response = make_api_request(f"projects/{project_id}", method="PUT", data=data)
        
        if response and response.get('status') == 'success':
            click.echo(f"{Fore.GREEN}Project visibility updated successfully to {visibility}.{Style.RESET_ALL}")
            
            if visibility == "public":
                click.echo(f"{Fore.GREEN}Your project is now publicly accessible to anyone.{Style.RESET_ALL}")
            elif visibility == "private":
                click.echo(f"{Fore.GREEN}Your project is now restricted to your IP address only.{Style.RESET_ALL}")
                click.echo(f"IP address: {', '.join(allowed_ips)}")
            elif visibility == "group":
                click.echo(f"{Fore.GREEN}Your project is now restricted to specific IP addresses.{Style.RESET_ALL}")
                click.echo(f"IP addresses: {', '.join(allowed_ips)}")
            
            # Update the local project data
            project['visibility'] = visibility
            project['allowed_ips'] = allowed_ips
            with open(current_project_path, "w") as f:
                json.dump(project, f)
            
            # Update .htaccess file to enforce IP restrictions
            from orbizen.utils.api import update_htaccess_for_visibility
            update_htaccess_for_visibility(project_id, visibility, allowed_ips)
            
        else:
            error_msg = response.get('message', 'Unknown error') if response else 'No response'
            click.echo(f"{Fore.RED}Error updating project visibility: {error_msg}{Style.RESET_ALL}")
    except Exception as e:
        click.echo(f"{Fore.RED}Error updating project visibility: {str(e)}{Style.RESET_ALL}") 