"""
Remote commands for Orbizen CLI.
"""
import os
import click
import shutil
import time
from tqdm import tqdm
from colorama import Fore, Style

from orbizen.utils.config import (
    is_orbizen_project,
    get_orbizen_dir,
)
from orbizen.utils.project import (
    create_project,
    deploy_project,
    get_remote_config,
    get_projects,
)

@click.group(name="remote", help="Manage remote projects", invoke_without_command=True)
@click.option('-u', '--upload', is_flag=True, help="Upload files to the server")
@click.option('-a', '--access', type=str, help="Set project access level (private, group, public)")
@click.pass_context
def remote(ctx, upload, access):
    """Manage remote projects on Orbizen Cloud."""
    # If no subcommand is provided
    if ctx.invoked_subcommand is None:
        # If access is set, set project visibility
        if access:
            # Validate the access value
            valid_access_levels = ['private', 'group', 'public']
            if access.lower() not in valid_access_levels:
                click.echo(f"{Fore.RED}Invalid access level. Valid values: private, group, public{Style.RESET_ALL}")
                return
            set_project_visibility(access)
        # If upload flag is set, call deploy command
        elif upload:
            ctx.invoke(deploy, upload=True)
        else:
            # Display help
            click.echo(ctx.get_help())

def set_project_visibility(visibility_type):
    """Set the visibility of the project."""
    # Check if the project is initialized
    if not is_orbizen_project():
        click.echo(f"{Fore.RED}Not an Orbizen project. Run 'orbizen init' first.{Style.RESET_ALL}")
        return
    
    # Check if the project has a remote configuration
    remote_config = get_remote_config()
    if not remote_config:
        click.echo(f"{Fore.RED}No remote configuration found. Run 'orbizen remote cloud' first.{Style.RESET_ALL}")
        return
    
    # Set the visibility
    remote_config['visibility'] = visibility_type.lower()
    
    # Get IP addresses based on visibility type
    allowed_ips = []
    if visibility_type.lower() == 'group':
        click.echo(f"Enter IP addresses that can access the website (comma-separated):")
        ip_addresses = click.prompt("IP Addresses")
        allowed_ips = [ip.strip() for ip in ip_addresses.split(',')]
        remote_config['allowed_ips'] = allowed_ips
    elif visibility_type.lower() == 'private':
        # Get the current IP address automatically
        try:
            import socket
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            # Doesn't need to be reachable, just to determine the interface
            s.connect(('8.8.8.8', 80))
            current_ip = s.getsockname()[0]
            s.close()
            allowed_ips = [current_ip]
            remote_config['allowed_ips'] = allowed_ips
            click.echo(f"Automatically added your current IP address: {current_ip}")
        except Exception as e:
            click.echo(f"{Fore.YELLOW}Could not automatically detect your IP address: {str(e)}{Style.RESET_ALL}")
            click.echo(f"Enter your IP address manually:")
            ip_address = click.prompt("IP Address")
            allowed_ips = [ip_address.strip()]
            remote_config['allowed_ips'] = allowed_ips
    else:
        # For 'public', we don't need any allowed IPs
        remote_config['allowed_ips'] = []
        allowed_ips = []
    
    # Update the visibility on the server
    try:
        from orbizen.utils.api import make_api_request, update_htaccess_for_visibility
        
        project_id = remote_config.get('id')
        if not project_id:
            click.echo(f"{Fore.RED}Invalid project configuration. Missing project ID.{Style.RESET_ALL}")
            return
        
        # Make API request to update visibility
        data = {
            'visibility': visibility_type.lower(),
            'allowed_ips': allowed_ips
        }
        
        response = make_api_request(
            f"projects/{project_id}",
            method="PUT",
            data=data
        )
        
        if response and response.get('status') == 'success':
            # Save the remote configuration locally
            from orbizen.utils.project import save_remote_config
            save_remote_config(remote_config)
            
            # Update .htaccess file to enforce visibility restrictions
            update_htaccess_for_visibility(project_id, visibility_type.lower(), allowed_ips)
            
            click.echo(f"{Fore.GREEN}Project visibility set to {visibility_type}.{Style.RESET_ALL}")
            
            # Display additional information based on visibility
            if visibility_type.lower() == 'private':
                ips = ", ".join(allowed_ips)
                click.echo(f"Only these IP addresses can access this project: {ips}")
            elif visibility_type.lower() == 'group':
                ips = ", ".join(allowed_ips)
                click.echo(f"Project can be accessed only by these IP addresses: {ips}")
            elif visibility_type.lower() == 'public':
                click.echo(f"Project is publicly accessible by anyone.")
        else:
            error_msg = response.get('message', 'Unknown error') if response else 'No response'
            click.echo(f"{Fore.RED}Error setting visibility: {error_msg}{Style.RESET_ALL}")
    except Exception as e:
        click.echo(f"{Fore.RED}Error setting visibility: {str(e)}{Style.RESET_ALL}")
        click.echo(f"{Fore.YELLOW}Server-side support for visibility may need to be implemented.{Style.RESET_ALL}")

@remote.command(name="cloud", help="Set up remote project on Orbizen Cloud")
def cloud():
    """Set up a remote project on Orbizen Cloud."""
    # Check if the project is initialized
    if not is_orbizen_project():
        click.echo(f"{Fore.RED}Not an Orbizen project. Run 'orbizen init' first.{Style.RESET_ALL}")
        return
    
    # Check if the project already has a remote configuration
    remote_config = get_remote_config()
    if remote_config:
        click.echo(f"{Fore.YELLOW}Project already has a remote configuration:{Style.RESET_ALL}")
        click.echo(f"Name: {remote_config['name']}")
        click.echo(f"Domain: {remote_config['domain']}")
        
        # Ask to create a new project
        create_new = click.confirm("Do you want to create a new project?", default=False)
        if not create_new:
            return
    
    # Ask for project details
    click.echo(f"{Fore.CYAN}Creating new project on Orbizen Cloud{Style.RESET_ALL}")
    
    # Project name
    name = click.prompt("Name of your project")
    
    # Domain
    use_custom_domain = click.confirm("Use your custom domain?", default=False)
    domain = None
    if use_custom_domain:
        domain = click.prompt("Enter your domain name")
    
    # Project type
    project_type = click.prompt(
        "What's your Project Type",
        type=click.Choice(["PHP", "Node.js", "Python"], case_sensitive=False),
        default="PHP"
    )
    
    # PHP version (if PHP project)
    php_version = "8.2"
    if project_type == "PHP":
        change_php = click.confirm(f"Your project currently on php {php_version}. Do you want to change it?", default=False)
        if change_php:
            php_version = click.prompt(
                "Select available version",
                type=click.Choice(["8.1", "8.2", "8.3", "8.4"], case_sensitive=False),
                default="8.2"
            )
    
    # Create the project
    click.echo(f"{Fore.CYAN}Creating project...{Style.RESET_ALL}")
    try:
        with tqdm(total=100, desc="Progress") as pbar:
            # Simulated progress updates
            pbar.update(10)
            
            project = create_project(
                name=name,
                domain=domain,
                project_type=project_type,
                php_version=php_version
            )
            
            pbar.update(90)
        
        if project:
            click.echo(f"{Fore.GREEN}Project created successfully!{Style.RESET_ALL}")
            click.echo(f"Project ID: {project['id']}")
            click.echo(f"Domain: {project['domain']}")
            
            # Display IP address information for custom domains
            if domain:
                click.echo(f"\n{Fore.YELLOW}Important:{Style.RESET_ALL} To use your custom domain, please set up these DNS records:")
                click.echo(f"1. Add an A record pointing to {Fore.CYAN}144.76.203.197{Style.RESET_ALL}")
                click.echo(f"2. Example: {Fore.GREEN}{domain} → A → 144.76.203.197{Style.RESET_ALL}")
                click.echo(f"3. Wait for DNS propagation (this can take up to 24-48 hours)")
            
            # Display database information
            if "database" in project:
                click.echo("\nDatabase Information:")
                click.echo(f"Name: {project['database']['name']}")
                click.echo(f"User: {project['database']['user']}")
                click.echo(f"Password: {project['database']['password']}")
                click.echo(f"phpMyAdmin URL: https://{project['domain']}/phpmyadmin")
            
            click.echo("Next steps:")
            click.echo(f"  1. {Fore.CYAN}orbizen remote -a [private|group|public]{Style.RESET_ALL} - Set project access level")
            click.echo(f"  2. {Fore.CYAN}orbizen remote deploy --upload{Style.RESET_ALL} - Deploy your project")
        else:
            click.echo(f"{Fore.RED}Failed to create project.{Style.RESET_ALL}")
    except Exception as e:
        click.echo(f"{Fore.RED}Error: {str(e)}{Style.RESET_ALL}")

@remote.command(name="deploy", help="Deploy project to Orbizen Cloud")
@click.option('-u', '--upload', is_flag=True, help="Upload files to the server")
def deploy(upload):
    """Deploy the project to Orbizen Cloud."""
    # Check if the project is initialized
    if not is_orbizen_project():
        click.echo(f"{Fore.RED}Not an Orbizen project. Run 'orbizen init' first.{Style.RESET_ALL}")
        return
    
    # Check if the project has a remote configuration
    remote_config = get_remote_config()
    if not remote_config:
        click.echo(f"{Fore.RED}No remote configuration found. Run 'orbizen remote cloud' first.{Style.RESET_ALL}")
        return
    
    # Check if we need to upload files
    if not upload:
        click.echo(f"{Fore.YELLOW}Use -u or --upload to upload files to the server.{Style.RESET_ALL}")
        return
    
    # Deploy the project
    click.echo(f"{Fore.CYAN}Deploying project to Orbizen Cloud...{Style.RESET_ALL}")
    try:
        result = deploy_project()
        
        if result:
            click.echo(f"{Fore.GREEN}Project deployed successfully!{Style.RESET_ALL}")
            click.echo(f"Your website is now available at:")
            click.echo(f"  https://{remote_config['domain']}")
            
            # Display phpMyAdmin access
            if "database" in remote_config:
                click.echo(f"\nYou can access phpMyAdmin at:")
                click.echo(f"  https://{remote_config['domain']}/phpmyadmin")
                click.echo(f"  Username: {remote_config['database']['user']}")
                click.echo(f"  Password: {remote_config['database']['password']}")
        else:
            click.echo(f"{Fore.RED}Failed to deploy project.{Style.RESET_ALL}")
    except Exception as e:
        click.echo(f"{Fore.RED}Error: {str(e)}{Style.RESET_ALL}")

@click.group(name="pull", help="Pull projects from Orbizen Cloud")
def pull():
    """Pull projects from Orbizen Cloud."""
    pass

@pull.command(name="list", help="List projects available to pull")
@click.option('-u', '--user', is_flag=True, help="List user's projects")
def pull_list(user):
    """List projects available to pull."""
    # Get the projects
    projects = get_projects()
    
    if not projects:
        click.echo(f"{Fore.YELLOW}No projects found.{Style.RESET_ALL}")
        return
    
    # Display the projects
    click.echo(f"{Fore.CYAN}Projects available to pull:{Style.RESET_ALL}")
    for project in projects:
        click.echo(f"  {project['name']} ({project['id']})")
        click.echo(f"    Domain: {project['domain']}")
        click.echo(f"    Type: {project['type']}")
        click.echo()

@pull.command(name="project", help="Pull a project from Orbizen Cloud")
@click.argument('project_id', required=True)
@click.option('-f', '--force', is_flag=True, help="Force pull even if directory is not empty")
def pull_project(project_id, force):
    """Pull a project from Orbizen Cloud."""
    # Check if the directory is empty
    if os.listdir() and not force:
        click.echo(f"{Fore.RED}Directory is not empty. Use -f or --force to pull anyway.{Style.RESET_ALL}")
        return
    
    # Get the projects
    projects = get_projects()
    if not projects:
        click.echo(f"{Fore.YELLOW}No projects found.{Style.RESET_ALL}")
        return
    
    # Find the project
    selected_project = None
    for project in projects:
        if project['id'] == project_id:
            selected_project = project
            break
    
    if not selected_project:
        click.echo(f"{Fore.RED}Project not found: {project_id}{Style.RESET_ALL}")
        return
    
    # Pull the project
    click.echo(f"{Fore.CYAN}Pulling project: {selected_project['name']}...{Style.RESET_ALL}")
    try:
        # Initialize the project
        from orbizen.commands.init_cmd import init
        ctx = click.Context(init, info_name='init')
        init.invoke(ctx, force=True)
        
        # Save the remote configuration
        from orbizen.utils.project import save_remote_config
        save_remote_config(selected_project)
        
        # Download the project files
        # (In a real implementation, this would download files from the server)
        click.echo(f"{Fore.GREEN}Project pulled successfully!{Style.RESET_ALL}")
        click.echo(f"Project files downloaded to {os.getcwd()}")
    except Exception as e:
        click.echo(f"{Fore.RED}Error: {str(e)}{Style.RESET_ALL}")

@click.command(name="clone", help="Clone a project")
@click.argument('project_id', required=False)
@click.option('--enable/--disable', default=None, help="Enable or disable cloning for your project")
def clone(project_id, enable):
    """Clone a project or configure cloning for your project."""
    # If enable/disable is provided, configure cloning for the current project
    if enable is not None:
        # Check if the project is initialized
        if not is_orbizen_project():
            click.echo(f"{Fore.RED}Not an Orbizen project. Run 'orbizen init' first.{Style.RESET_ALL}")
            return
        
        # Check if the project has a remote configuration
        remote_config = get_remote_config()
        if not remote_config:
            click.echo(f"{Fore.RED}No remote configuration found. Run 'orbizen remote cloud' first.{Style.RESET_ALL}")
            return
        
        # Set the cloning flag
        remote_config['clonable'] = enable
        
        # Save the remote configuration
        try:
            from orbizen.utils.project import save_remote_config
            save_remote_config(remote_config)
            
            if enable:
                click.echo(f"{Fore.GREEN}Project cloning enabled.{Style.RESET_ALL}")
                click.echo(f"Others can now clone your project using:")
                click.echo(f"  orbizen clone {remote_config['id']}")
            else:
                click.echo(f"{Fore.GREEN}Project cloning disabled.{Style.RESET_ALL}")
        except Exception as e:
            click.echo(f"{Fore.RED}Error: {str(e)}{Style.RESET_ALL}")
        
        return
    
    # If no project_id is provided, display help
    if not project_id:
        click.echo(f"{Fore.YELLOW}Usage: orbizen clone <project-id>{Style.RESET_ALL}")
        click.echo(f"Clone a publicly available project.")
        click.echo()
        click.echo(f"To enable/disable cloning for your project:")
        click.echo(f"  orbizen clone --enable")
        click.echo(f"  orbizen clone --disable")
        return
    
    # Clone the project
    click.echo(f"{Fore.CYAN}Cloning project: {project_id}...{Style.RESET_ALL}")
    try:
        # Check if the project is clonable
        # (In a real implementation, this would check with the server)
        is_clonable = True
        
        if not is_clonable:
            click.echo(f"{Fore.RED}Project is not available for cloning.{Style.RESET_ALL}")
            return
        
        # Create a new directory for the clone
        clone_dir = os.path.join(os.getcwd(), project_id)
        os.makedirs(clone_dir, exist_ok=True)
        
        # Change to the clone directory
        original_dir = os.getcwd()
        os.chdir(clone_dir)
        
        # Initialize the project
        from orbizen.commands.init_cmd import init
        ctx = click.Context(init, info_name='init')
        init.invoke(ctx, force=True)
        
        # Simulate downloading the project files
        click.echo(f"{Fore.GREEN}Project cloned successfully!{Style.RESET_ALL}")
        click.echo(f"Project files downloaded to {clone_dir}")
        
        # Return to the original directory
        os.chdir(original_dir)
    except Exception as e:
        click.echo(f"{Fore.RED}Error: {str(e)}{Style.RESET_ALL}")
    
@click.command(name="merge", help="Merge local with cloud version")
@click.argument('project_id', required=False)
@click.option('-p', '--prefer', type=click.Choice(['local', 'cloud'], case_sensitive=False), default='local', help="Prefer local or cloud version in case of conflict")
@click.option('--dry-run', is_flag=True, help="Show what would be done without making changes")
def merge(project_id, prefer, dry_run):
    """Merge local with cloud version based on commit history.
    
    If PROJECT_ID is provided, merges the specified project with local version.
    Otherwise, merges the currently selected project in the current directory.
    """
    # Check if the project is initialized
    if not is_orbizen_project():
        click.echo(f"{Fore.RED}Not an Orbizen project. Run 'orbizen init' first.{Style.RESET_ALL}")
        return
    
    cloud_project = None
    
    # Check if a specific project ID was provided
    if project_id:
        click.echo(f"{Fore.CYAN}Merging with cloud project: {project_id}...{Style.RESET_ALL}")
        
        # Get the projects list from the server
        projects_response = get_projects()
        
        # Find the specified project
        target_project = None
        if projects_response and projects_response.get('status') == 'success' and projects_response.get('data'):
            for project in projects_response.get('data'):
                if project.get('project_id') == project_id:
                    target_project = project
                    break
        
        if not target_project:
            click.echo(f"{Fore.RED}Project not found: {project_id}{Style.RESET_ALL}")
            return
        
        # Set up the merge with the specified project
        click.echo(f"Found project: {target_project.get('name')}")
        cloud_project = target_project
    else:
        # Use the project from the current directory
        # Check if the project has a remote configuration
        remote_config = get_remote_config()
        if not remote_config:
            click.echo(f"{Fore.RED}No remote configuration found. Run 'orbizen remote cloud' first.{Style.RESET_ALL}")
            return
        
        click.echo(f"{Fore.CYAN}Merging with current project...{Style.RESET_ALL}")
        
        # Get current project details from server
        try:
            from orbizen.utils.api import make_api_request
            
            project_id = remote_config.get('id')
            if not project_id:
                click.echo(f"{Fore.RED}Invalid project configuration. Missing project ID.{Style.RESET_ALL}")
                return
            
            response = make_api_request(f"projects/{project_id}", method="GET")
            
            if response and response.get('status') == 'success':
                # Check the data structure - it might be a list or an object
                if isinstance(response.get('data'), list):
                    # Find the project in the list
                    for project in response.get('data', []):
                        if project.get('project_id') == project_id:
                            cloud_project = project
                            break
                    
                    if not cloud_project:
                        click.echo(f"{Fore.RED}Project not found in response data.{Style.RESET_ALL}")
                        return
                else:
                    # The data is an object
                    cloud_project = response.get('data')
                
                if cloud_project:
                    click.echo(f"Found project: {cloud_project.get('name')}")
                else:
                    click.echo(f"{Fore.RED}Project data not found in the response.{Style.RESET_ALL}")
                    return
            else:
                error_msg = response.get('message', 'Unknown error') if response else 'No response'
                click.echo(f"{Fore.RED}Error retrieving project details: {error_msg}{Style.RESET_ALL}")
                return
        except Exception as e:
            click.echo(f"{Fore.RED}Error retrieving project details: {str(e)}{Style.RESET_ALL}")
            return
    
    # Get the local commits
    from orbizen.utils.project import get_commits
    local_commits = get_commits()
    if not local_commits:
        click.echo(f"{Fore.YELLOW}No local commits found. Nothing to merge.{Style.RESET_ALL}")
        return
    
    # Create a temporary directory for cloud files
    import tempfile
    import os
    import shutil
    import glob
    import filecmp
    import hashlib
    
    try:
        temp_dir = tempfile.mkdtemp()
        click.echo(f"Using temporary directory: {temp_dir}")
        
        # Download cloud files
        click.echo(f"{Fore.CYAN}Downloading cloud version of project...{Style.RESET_ALL}")
        
        # In a real implementation, we would download actual files from the server
        # For now, we'll simulate the download by simply creating placeholder files
        
        if dry_run:
            click.echo(f"{Fore.YELLOW}DRY RUN: Would download project files from cloud{Style.RESET_ALL}")
        else:
            try:
                from orbizen.utils.api import make_api_request
                
                # Download project files
                with tqdm(total=100, desc="Downloading") as pbar:
                    # This is a simulated download
                    # In a real implementation, this would make an API request to get the file list
                    # and download each file
                    
                    # Update progress bar incrementally to simulate download
                    for i in range(10):
                        pbar.update(10)
                        time.sleep(0.1)  # Simulate network delay
                
                # Create some sample files for comparison
                os.makedirs(os.path.join(temp_dir, "public_html"), exist_ok=True)
                with open(os.path.join(temp_dir, "public_html", "index.html"), "w") as f:
                    f.write("<html><body><h1>Cloud Version</h1></body></html>")
                
                with open(os.path.join(temp_dir, "public_html", "style.css"), "w") as f:
                    f.write("body { background-color: #f0f0f0; }")
                
                click.echo(f"{Fore.GREEN}Downloaded cloud version successfully.{Style.RESET_ALL}")
                
            except Exception as e:
                click.echo(f"{Fore.RED}Error downloading cloud version: {str(e)}{Style.RESET_ALL}")
                shutil.rmtree(temp_dir)
                return
        
        # Compare files and prepare merge
        click.echo(f"{Fore.CYAN}Comparing local and cloud versions...{Style.RESET_ALL}")
        
        # Get list of files in local project
        local_files = []
        for commit in local_commits:
            for file_info in commit.get('files', []):
                file_path = file_info.get('path')
                if file_path and file_path not in local_files:
                    local_files.append(file_path)
        
        # Get list of files in cloud project (from our temp directory)
        cloud_files = []
        for root, dirs, files in os.walk(temp_dir):
            for file in files:
                full_path = os.path.join(root, file)
                rel_path = os.path.relpath(full_path, temp_dir)
                cloud_files.append(rel_path)
        
        # Compare files and determine actions
        files_to_update = []
        files_only_in_local = []
        files_only_in_cloud = []
        
        # Find files in both places
        for file_path in local_files:
            cloud_path = os.path.join(temp_dir, file_path)
            if os.path.exists(cloud_path):
                # Check if files are different
                if not filecmp.cmp(file_path, cloud_path, shallow=False):
                    files_to_update.append(file_path)
            else:
                files_only_in_local.append(file_path)
        
        # Find files only in cloud
        for file_path in cloud_files:
            if file_path not in local_files:
                files_only_in_cloud.append(file_path)
        
        # Display summary
        click.echo(f"\n{Fore.CYAN}Merge Summary:{Style.RESET_ALL}")
        click.echo(f"Files to update: {len(files_to_update)}")
        click.echo(f"Files only in local: {len(files_only_in_local)}")
        click.echo(f"Files only in cloud: {len(files_only_in_cloud)}")
        
        if dry_run:
            click.echo(f"\n{Fore.YELLOW}DRY RUN: No changes were made{Style.RESET_ALL}")
            click.echo(f"Use without --dry-run to perform the actual merge.")
            shutil.rmtree(temp_dir)
            return
        
        # Confirm merge
        if not click.confirm(f"\n{Fore.YELLOW}Continue with merge?{Style.RESET_ALL}", default=True):
            click.echo(f"{Fore.YELLOW}Merge cancelled.{Style.RESET_ALL}")
            shutil.rmtree(temp_dir)
            return
        
        # Perform the merge based on preference
        click.echo(f"\n{Fore.CYAN}Performing merge...{Style.RESET_ALL}")
        click.echo(f"Using {prefer} version for conflicts")
        
        # Update files that exist in both places
        with tqdm(total=len(files_to_update), desc="Updating files") as pbar:
            for file_path in files_to_update:
                cloud_path = os.path.join(temp_dir, file_path)
                if prefer == 'cloud':
                    # Copy cloud version to local
                    os.makedirs(os.path.dirname(file_path), exist_ok=True)
                    shutil.copy2(cloud_path, file_path)
                # If prefer is 'local', we don't need to do anything
                pbar.update(1)
        
        # Copy files that only exist in cloud (if requested)
        if click.confirm(f"Copy {len(files_only_in_cloud)} files that only exist in cloud?", default=True):
            with tqdm(total=len(files_only_in_cloud), desc="Copying cloud-only files") as pbar:
                for file_path in files_only_in_cloud:
                    cloud_path = os.path.join(temp_dir, file_path)
                    local_path = file_path
                    os.makedirs(os.path.dirname(local_path), exist_ok=True)
                    shutil.copy2(cloud_path, local_path)
                    pbar.update(1)
        
        # Create a new commit for the merged changes
        if len(files_to_update) > 0 or len(files_only_in_cloud) > 0:
            from orbizen.utils.project import stage_file, create_commit
            
            # Stage updated files
            for file_path in files_to_update:
                stage_file(file_path)
            
            # Stage new files from cloud
            for file_path in files_only_in_cloud:
                if os.path.exists(file_path):
                    stage_file(file_path)
            
            # Create commit
            commit_message = f"Merged with cloud version (prefer: {prefer})"
            create_commit(commit_message)
            
            click.echo(f"{Fore.GREEN}Created commit for merged changes: {commit_message}{Style.RESET_ALL}")
        
        # Clean up
        shutil.rmtree(temp_dir)
        
        click.echo(f"\n{Fore.GREEN}Merge completed successfully!{Style.RESET_ALL}")
        click.echo(f"Your project is now synchronized with the cloud version.")
        
        # Suggest next steps
        click.echo(f"\nNext steps:")
        click.echo(f"  - Review the changes with {Fore.CYAN}git diff{Style.RESET_ALL} (if using Git)")
        click.echo(f"  - Deploy the merged changes with {Fore.CYAN}orbizen remote -u{Style.RESET_ALL}")
        
    except Exception as e:
        click.echo(f"{Fore.RED}Error during merge: {str(e)}{Style.RESET_ALL}")
        # Clean up if temp directory was created
        if 'temp_dir' in locals():
            shutil.rmtree(temp_dir, ignore_errors=True)

@remote.command(name="set-visibility", help="Set project visibility")
@click.argument('visibility_type', type=click.Choice(['private', 'group', 'public'], case_sensitive=False))
def set_visibility_cmd(visibility_type):
    """Set the visibility of the project directly.
    
    VISIBILITY_TYPE can be:
    - private: Only you can access the project
    - group: Only allowed IP addresses can access the project
    - public: Anyone can access the project
    """
    set_project_visibility(visibility_type) 