"""
Status command for Orbizen CLI.
"""
import os
import json
import click
import requests
from datetime import datetime
from tabulate import tabulate
from colorama import Fore, Style

from orbizen.utils.config import get_version
from orbizen.utils.credentials import get_global_credentials, get_projects_for_user, is_authenticated

@click.command(name="status", help="Show system status and information")
def status():
    """Show system status and information."""
    click.echo(f"{Fore.CYAN}Orbizen Cloud CLI v{get_version()}{Style.RESET_ALL}")
    click.echo("Status information:")
    
    # Authentication status
    if is_authenticated():
        credentials = get_global_credentials()
        email = credentials.get('email')
        user_id = credentials.get('user_id')
        created_at = credentials.get('created_at')
        
        login_time = None
        if created_at:
            try:
                # Parse ISO format datetime
                login_dt = datetime.fromisoformat(created_at)
                login_time = login_dt.strftime("%Y-%m-%d %H:%M:%S")
            except:
                login_time = "Unknown"
        
        click.echo(f"  Authentication: {Fore.GREEN}Logged in{Style.RESET_ALL}")
        click.echo(f"  User: {email}")
        click.echo(f"  Login time: {login_time or 'Unknown'}")
        
        # Get user's projects
        projects = get_projects_for_user()
        if projects:
            click.echo("\nYour projects:")
            
            # Format project data for table display
            table_data = []
            for project in projects:
                table_data.append([
                    project.get('name', 'Unknown'),
                    project.get('domain', 'Unknown'),
                    project.get('type', 'Unknown'),
                    project.get('php_version', 'Unknown'),
                    project.get('created_at', 'Unknown')
                ])
            
            # Print table of projects
            headers = ["Name", "Domain", "Type", "PHP Version", "Created"]
            click.echo(tabulate(table_data, headers=headers, tablefmt="simple"))
        else:
            click.echo("\nYou don't have any projects yet.")
            click.echo(f"Create one with: {Fore.CYAN}orbizen remote cloud{Style.RESET_ALL}")
    else:
        click.echo(f"  Authentication: {Fore.YELLOW}Not logged in{Style.RESET_ALL}")
        click.echo(f"  Run '{Fore.CYAN}orbizen login{Style.RESET_ALL}' to authenticate.")
    
    # API status
    try:
        response = requests.get("https://orbizen.cloud/api/status.php", timeout=5)
        if response.status_code == 200:
            try:
                data = response.json()
                click.echo(f"\nServer status: {Fore.GREEN}Online{Style.RESET_ALL}")
                if "version" in data:
                    click.echo(f"  API version: {data['version']}")
                if "message" in data:
                    click.echo(f"  Message: {data['message']}")
            except:
                click.echo(f"\nServer status: {Fore.GREEN}Online{Style.RESET_ALL}")
        else:
            click.echo(f"\nServer status: {Fore.YELLOW}Online but not responding properly{Style.RESET_ALL}")
    except:
        click.echo(f"\nServer status: {Fore.RED}Offline or unreachable{Style.RESET_ALL}")
    
    # Local configuration
    click.echo("\nLocal configuration:")
    config_dir = os.path.expanduser("~/.orbizen")
    click.echo(f"  Config directory: {config_dir}")
    
    if os.path.isdir(config_dir):
        config_files = [f for f in os.listdir(config_dir) if os.path.isfile(os.path.join(config_dir, f))]
        if config_files:
            click.echo(f"  Config files: {', '.join(config_files)}")
        else:
            click.echo("  No configuration files found.")
    else:
        click.echo("  Configuration directory does not exist.") 