"""
API utilities for Orbizen CLI.
"""
import os
import json
import requests
import urllib.parse
import hashlib
import base64
import time
from datetime import datetime, timezone
from requests.exceptions import RequestException
from colorama import Fore, Style
import shutil
from tqdm import tqdm

from orbizen.utils.config import get_global_credentials, save_global_credentials

# WHM/cPanel API Constants
WHM_API_ENDPOINT = "https://orbizen.cloud:2087/json-api/"
WHM_API_KEY = "ZM37RF7PG4XQHG2CUSG2LFYZ7I6DJXSB"
CPANEL_API_ENDPOINT = "https://orbizen.cloud:2083/execute/"
BASE_PROJECT_DIR = "/home/orbizen/orbizen.cloud/cli/project"
PHPMYADMIN_TEMPLATE_DIR = "/home/orbizen/orbizen.cloud/phpmyadmin"

# Orbizen API Constants
API_ENDPOINT = "https://orbizen.cloud/api/"

class APIError(Exception):
    """Exception raised for API errors."""
    pass

def is_token_expired(credentials):
    """Check if the token is expired."""
    if not credentials or 'created_at' not in credentials:
        return True
        
    try:
        # Parse timestamp from ISO format
        created_at = datetime.fromisoformat(credentials['created_at'])
        # Calculate expiration time (24 hours after creation)
        expires_at = created_at.timestamp() + 86400  # 24 hours in seconds
        current_time = datetime.now().timestamp()
        
        # Debug output
        print(f"{Fore.BLUE}Token info: Created {(current_time - created_at.timestamp()) / 3600:.1f} hours ago{Style.RESET_ALL}")
        
        return current_time > expires_at
    except Exception as e:
        print(f"{Fore.YELLOW}Warning: Could not check token expiration: {str(e)}{Style.RESET_ALL}")
        return True

def refresh_token():
    """Attempt to refresh the authentication token."""
    credentials = get_global_credentials()
    if not credentials:
        print(f"{Fore.RED}No credentials found. Please login first.{Style.RESET_ALL}")
        return False
        
    try:
        # Make a request to refresh the token
        refresh_url = f"{API_ENDPOINT}refresh.php"
        response = requests.post(
            refresh_url,
            json={"user_id": credentials.get('user_id')},
            headers={"Authorization": f"Bearer {credentials.get('token')}"}
        )
        
        if response.status_code == 200:
            data = response.json()
            if data.get('status') == 'success' and 'token' in data.get('data', {}):
                # Update credentials with the new token
                credentials['token'] = data['data']['token']
                credentials['created_at'] = datetime.now().isoformat()
                save_global_credentials(credentials)
                print(f"{Fore.GREEN}Token refreshed successfully.{Style.RESET_ALL}")
                return True
                
        print(f"{Fore.YELLOW}Token refresh failed. Please login again.{Style.RESET_ALL}")
        return False
    except Exception as e:
        print(f"{Fore.RED}Error refreshing token: {str(e)}{Style.RESET_ALL}")
        return False

def get_headers():
    """Get the headers for API requests."""
    credentials = get_global_credentials()
    if not credentials:
        return {}
    
    # Check if token is expired
    if is_token_expired(credentials):
        print(f"{Fore.YELLOW}Token appears to be expired. Attempting to refresh...{Style.RESET_ALL}")
        if not refresh_token():
            print(f"{Fore.YELLOW}Please run 'orbizen login' to get a new token.{Style.RESET_ALL}")
            # We'll still try with the existing token, it might still work
    
    return {
        "Authorization": f"Bearer {credentials.get('token')}",
        "Content-Type": "application/json",
    }

def make_api_request(endpoint, method="GET", data=None, files=None):
    """Make a request to the Orbizen API."""
    url = f"{API_ENDPOINT}{endpoint}"
    headers = get_headers()
    
    # Debug output
    credentials = get_global_credentials()
    if not credentials or not credentials.get('token'):
        print(f"{Fore.YELLOW}Warning: No valid credentials found. Make sure you are logged in.{Style.RESET_ALL}")
        print(f"{Fore.YELLOW}Run 'orbizen login' to authenticate.{Style.RESET_ALL}")
    
    # Add user_id parameter to URL for fallback authentication
    if "?" in url:
        url += f"&user_id={credentials.get('user_id') if credentials else ''}"
    else:
        url += f"?user_id={credentials.get('user_id') if credentials else ''}"
    
    print(f"Making API request to: {url}")
    print(f"Auth header present: {'Yes' if headers.get('Authorization') else 'No'}")
    
    try:
        # For file uploads with POST, we need to adjust headers
        if method == "POST" and files:
            # Remove Content-Type for multipart/form-data (will be set automatically)
            file_headers = headers.copy()
            if "Content-Type" in file_headers:
                del file_headers["Content-Type"]
        else:
            file_headers = headers
        
        # Make the request
        max_retries = 3
        retry_count = 0
        
        while retry_count < max_retries:
            if method == "GET":
                response = requests.get(url, headers=headers)
            elif method == "POST":
                if files:
                    response = requests.post(url, headers=file_headers, files=files)
                else:
                    response = requests.post(url, json=data, headers=headers)
            elif method == "PUT":
                response = requests.put(url, json=data, headers=headers)
            elif method == "DELETE":
                response = requests.delete(url, headers=headers)
            else:
                raise APIError(f"Unsupported HTTP method: {method}")
            
            # Print response status for debugging
            print(f"Response status: {response.status_code}")
            
            # Debug: Print raw response text
            print(f"Raw response content: {response.text[:200]}{'...' if len(response.text) > 200 else ''}")
            
            # Detailed error logging
            if response.status_code == 401:
                print(f"{Fore.RED}Authentication failed (401 Unauthorized){Style.RESET_ALL}")
                print(f"Response headers: {dict(response.headers)}")
                print(f"Response body: {response.text}")
                
                # Try to refresh token on the first 401
                if retry_count == 0 and refresh_token():
                    print(f"{Fore.GREEN}Token refreshed, retrying request...{Style.RESET_ALL}")
                    headers = get_headers()  # Get updated headers with new token
                    retry_count += 1
                    continue
                
                # Try alternate auth method for deploy requests
                if "deploy" in endpoint and retry_count == 1:
                    print(f"{Fore.YELLOW}Trying alternative authentication for deployment...{Style.RESET_ALL}")
                    
                    # Add token as a query parameter as fallback
                    if "?" in url:
                        url += f"&token={credentials.get('token') if credentials else ''}"
                    else:
                        url += f"?token={credentials.get('token') if credentials else ''}"
                    
                    retry_count += 1
                    continue
            
            # Break the loop if we get a non-401 response or reached max retries
            break
        
        # Check for errors
        if response.status_code >= 400:
            print(f"Error response: {response.text}")
            response.raise_for_status()
        
        # Handle empty responses
        if not response.text.strip():
            print(f"{Fore.YELLOW}Warning: Empty response received from the server{Style.RESET_ALL}")
            return {"status": "error", "message": "Server returned an empty response"}
            
        try:
            # Try to parse JSON response
            return response.json()
        except ValueError:
            # Return raw text if JSON parsing fails
            print(f"{Fore.YELLOW}Warning: Could not parse JSON response. Raw response: {response.text[:200]}{Style.RESET_ALL}")
            return {"status": "error", "message": f"Invalid JSON response: {response.text[:100]}"}
    except RequestException as e:
        raise APIError(f"API request failed: {str(e)}")

def make_whm_request(function, params=None, method="GET"):
    """Make a request to the WHM API."""
    if params is None:
        params = {}
    
    # Build the URL
    url = f"{WHM_API_ENDPOINT}{function}"
    
    # Build the headers
    headers = {
        "Authorization": f"WHM {os.environ.get('WHM_USERNAME', 'root')}:{WHM_API_KEY}",
    }
    
    if method == "POST" and any(key.startswith("cpanel_jsonapi_") for key in params):
        headers["Content-Type"] = "application/x-www-form-urlencoded"
    
    try:
        if method == "GET":
            response = requests.get(url, params=params, headers=headers)
        elif method == "POST":
            if any(key.startswith("cpanel_jsonapi_") for key in params):
                response = requests.post(url, data=urllib.parse.urlencode(params), headers=headers)
            else:
                response = requests.post(url, json=params, headers=headers)
        else:
            raise APIError(f"Unsupported HTTP method: {method}")
        
        # Check for errors
        response.raise_for_status()
        
        return response.json()
    except RequestException as e:
        raise APIError(f"WHM API request failed: {str(e)}")

def create_project_directory(project_id):
    """Create a project directory on the server."""
    # Create project paths
    project_dir = f"{BASE_PROJECT_DIR}/{project_id}"
    public_html_dir = f"{project_dir}/public_html"
    
    # Create the main project directory
    mkdir_project_params = {
        "cpanel_jsonapi_user": "orbizen",
        "cpanel_jsonapi_apiversion": "2",
        "cpanel_jsonapi_module": "Fileman",
        "cpanel_jsonapi_func": "mkdir",
        "path": BASE_PROJECT_DIR,
        "name": project_id
    }
    
    make_whm_request("cpanel", mkdir_project_params, method="POST")
    
    # Create public_html directory
    mkdir_html_params = {
        "cpanel_jsonapi_user": "orbizen",
        "cpanel_jsonapi_apiversion": "2",
        "cpanel_jsonapi_module": "Fileman",
        "cpanel_jsonapi_func": "mkdir",
        "path": project_dir,
        "name": "public_html"
    }
    
    make_whm_request("cpanel", mkdir_html_params, method="POST")
    
    # Create a default index.html file
    index_content = '''<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Orbizen Cloud - Project</title>
    <style>
        body { 
            font-family: Arial, sans-serif; 
            margin: 0; 
            padding: 40px; 
            background: #f4f7f9; 
            color: #333; 
            text-align: center;
        }
        .container { 
            width: 80%; 
            margin: 50px auto; 
            background: white; 
            padding: 30px; 
            border-radius: 8px; 
            box-shadow: 0 2px 10px rgba(0,0,0,0.1); 
        }
        h1 { color: #0066cc; font-size: 36px; }
        .success { 
            background: #d4edda; 
            color: #155724; 
            padding: 15px; 
            border-radius: 4px; 
            margin: 20px 0;
        }
    </style>
</head>
<body>
    <div class="container">
        <h1>Project Created Successfully</h1>
        <div class="success">
            Your Orbizen Cloud project is now live!
        </div>
        <p>This is the default index page. Replace it with your own content.</p>
        <p><a href="/cli/project/''' + project_id + '''/phpmyadmin">Access phpMyAdmin</a></p>
    </div>
</body>
</html>'''
    
    # First create an empty file using mkfile
    mkfile_params = {
        "cpanel_jsonapi_user": "orbizen",
        "cpanel_jsonapi_apiversion": "2",
        "cpanel_jsonapi_module": "Fileman",
        "cpanel_jsonapi_func": "mkfile",
        "path": public_html_dir,
        "name": "index.html"
    }
    
    mkfile_result = make_whm_request("cpanel", mkfile_params, method="POST")
    
    # Then write the content
    write_params = {
        "cpanel_jsonapi_user": "orbizen",
        "cpanel_jsonapi_apiversion": "2",
        "cpanel_jsonapi_module": "Fileman",
        "cpanel_jsonapi_func": "fileop",
        "op": "edit",
        "file": "index.html",
        "dir": public_html_dir,
        "content": index_content
    }
    
    write_result = make_whm_request("cpanel", write_params, method="POST")
    
    # No longer creating a public_html/.htaccess file
    # Instead, we create a comprehensive root .htaccess file
    
    # Create project_dir/.htaccess file (root level)
    root_htaccess = '''# Redirect to public_html
<IfModule mod_rewrite.c>
    RewriteEngine On
    RewriteRule ^$ public_html/ [L]
    RewriteCond %{REQUEST_FILENAME} !-f
    RewriteCond %{REQUEST_FILENAME} !-d
    RewriteRule ^(.*)$ public_html/$1 [L]
</IfModule>

# Prevent directory listing
Options -Indexes

# PHP settings for the entire project
<IfModule mod_php.c>
    php_flag display_errors Off
    php_value upload_max_filesize 64M
    php_value post_max_size 64M
    php_value max_execution_time 300
    php_value max_input_time 300
</IfModule>

# Security headers
<IfModule mod_headers.c>
    Header set X-Content-Type-Options "nosniff"
    Header set X-XSS-Protection "1; mode=block"
    Header set X-Frame-Options "SAMEORIGIN"
</IfModule>

# Prevent access to sensitive files
<FilesMatch "^([.]htaccess|[.]htpasswd|[.]git|[.]env|[.]config)">
    Order Allow,Deny
    Deny from all
</FilesMatch>
'''
    
    # Create the root .htaccess file
    mkfile_root_htaccess_params = {
        "cpanel_jsonapi_user": "orbizen",
        "cpanel_jsonapi_apiversion": "2",
        "cpanel_jsonapi_module": "Fileman",
        "cpanel_jsonapi_func": "mkfile",
        "path": project_dir,
        "name": ".htaccess"
    }
    
    mkfile_root_htaccess_result = make_whm_request("cpanel", mkfile_root_htaccess_params, method="POST")
    
    # Write root .htaccess content
    write_root_htaccess_params = {
        "cpanel_jsonapi_user": "orbizen",
        "cpanel_jsonapi_apiversion": "2",
        "cpanel_jsonapi_module": "Fileman",
        "cpanel_jsonapi_func": "fileop",
        "op": "edit",
        "file": ".htaccess",
        "dir": project_dir,
        "content": root_htaccess
    }
    
    write_root_htaccess_result = make_whm_request("cpanel", write_root_htaccess_params, method="POST")
    
    return {
        "directory_creation": "success",
        "index_file_creation": mkfile_result,
        "index_file_content": write_result,
        "root_htaccess_creation": mkfile_root_htaccess_result,
        "root_htaccess_content": write_root_htaccess_result
    }

def setup_phpmyadmin(project_id, db_name, db_user, db_pass):
    """Set up phpMyAdmin for a project."""
    # Create paths
    project_dir = f"{BASE_PROJECT_DIR}/{project_id}"
    phpmyadmin_dir = f"{project_dir}"
    
    # Create phpMyAdmin directory
    mkdir_phpmyadmin_params = {
        "cpanel_jsonapi_user": "orbizen",
        "cpanel_jsonapi_apiversion": "2",
        "cpanel_jsonapi_module": "Fileman",
        "cpanel_jsonapi_func": "mkdir",
        "path": project_dir,
        "name": "phpmyadmin"
    }
    
    make_whm_request("cpanel", mkdir_phpmyadmin_params, method="POST")
    
    # Copy the phpMyAdmin template files
    copy_params = {
        "cpanel_jsonapi_user": "orbizen",
        "cpanel_jsonapi_apiversion": "2",
        "cpanel_jsonapi_module": "Fileman",
        "cpanel_jsonapi_func": "fileop",
        "op": "copy",
        "sourcefiles": PHPMYADMIN_TEMPLATE_DIR,
        "destfiles": phpmyadmin_dir,
        "doubledecode": 1
    }
    
    copy_result = make_whm_request("cpanel", copy_params, method="POST")
    
    # Generate blowfish secret
    blowfish_secret = hashlib.sha256(os.urandom(32)).hexdigest()
    
    # Create config file content
    config_content = f'''<?php
$cfg['blowfish_secret'] = '{blowfish_secret}';
$i = 0;
$i++;
$cfg['Servers'][$i]['auth_type'] = 'cookie';
$cfg['Servers'][$i]['host'] = 'localhost';
$cfg['Servers'][$i]['compress'] = false;
$cfg['Servers'][$i]['AllowNoPassword'] = false;
$cfg['Servers'][$i]['user'] = '{db_user}';
$cfg['Servers'][$i]['password'] = '{db_pass}';
$cfg['Servers'][$i]['only_db'] = '{db_name}';
$cfg['UploadDir'] = '';
$cfg['SaveDir'] = '';
'''
    
    # First create the empty config file
    mkfile_config_params = {
        "cpanel_jsonapi_user": "orbizen",
        "cpanel_jsonapi_apiversion": "2",
        "cpanel_jsonapi_module": "Fileman",
        "cpanel_jsonapi_func": "mkfile",
        "path": phpmyadmin_dir,
        "name": "config.inc.php"
    }
    
    mkfile_config_result = make_whm_request("cpanel", mkfile_config_params, method="POST")
    
    # Then write the content
    write_config_params = {
        "cpanel_jsonapi_user": "orbizen",
        "cpanel_jsonapi_apiversion": "2",
        "cpanel_jsonapi_module": "Fileman",
        "cpanel_jsonapi_func": "fileop",
        "op": "edit",
        "file": "config.inc.php",
        "dir": phpmyadmin_dir,
        "content": config_content
    }
    
    write_config_result = make_whm_request("cpanel", write_config_params, method="POST")
    
    # Set permissions
    perm_params = {
        "cpanel_jsonapi_user": "orbizen",
        "cpanel_jsonapi_apiversion": "2",
        "cpanel_jsonapi_module": "Fileman",
        "cpanel_jsonapi_func": "fileop",
        "op": "chmod",
        "sourcefiles": f"{phpmyadmin_dir}/config.inc.php",
        "metadata": "0644",
        "doubledecode": 1
    }
    
    perm_result = make_whm_request("cpanel", perm_params, method="POST")
    
    return {
        "phpmyadmin_setup": "success", 
        "template_copy": copy_result,
        "config_file_creation": mkfile_config_result,
        "config_file_content": write_config_result,
        "permissions": perm_result
    }

def create_database(db_name, db_user, db_pass):
    """Create a database and user on the server using cPanel API."""
    # Use the correct cPanel API module and function names for database operations
    # Create the database
    create_db_params = {
        "cpanel_jsonapi_user": "orbizen",
        "cpanel_jsonapi_apiversion": "2",
        "cpanel_jsonapi_module": "MysqlFE",
        "cpanel_jsonapi_func": "createdb",
        "db": db_name,
    }
    
    db_result = make_whm_request("cpanel", create_db_params, method="POST")
    
    # Create the user
    create_user_params = {
        "cpanel_jsonapi_user": "orbizen",
        "cpanel_jsonapi_apiversion": "2",
        "cpanel_jsonapi_module": "MysqlFE",
        "cpanel_jsonapi_func": "createdbuser",
        "dbuser": db_user,
        "password": db_pass,
    }
    
    user_result = make_whm_request("cpanel", create_user_params, method="POST")
    
    # Grant privileges to the user
    grant_params = {
        "cpanel_jsonapi_user": "orbizen",
        "cpanel_jsonapi_apiversion": "2",
        "cpanel_jsonapi_module": "MysqlFE",
        "cpanel_jsonapi_func": "setdbuserprivileges",
        "dbuser": db_user,
        "db": db_name,
        "privileges": "ALL PRIVILEGES",
    }
    
    grant_result = make_whm_request("cpanel", grant_params, method="POST")
    
    return {
        "database": db_result,
        "user": user_result,
        "privileges": grant_result,
    }

def create_subdomain(project_id):
    """Create a subdomain for a project using cPanel API."""
    print(f"Creating subdomain: {project_id}.orbizen.cloud")
    
    # Update paths to match the deploy.php paths
    base_dir = "/home/orbizen"
    project_dir = f"{base_dir}/orbizen.cloud/cli/project/{project_id}"
    public_html_dir = f"{project_dir}/public_html"
    
    # Parameters for the cPanel API call to create the subdomain
    params = {
        "cpanel_jsonapi_user": "orbizen",
        "cpanel_jsonapi_apiversion": "2",
        "cpanel_jsonapi_module": "SubDomain",
        "cpanel_jsonapi_func": "addsubdomain",
        "subdomain": project_id,
        "domain": "orbizen.cloud",
        "rootdomain": "orbizen.cloud",
        "dir": project_dir,  # Updated path to match deploy.php
        "disallowdot": 0
    }
    
    # Create the subdomain in cPanel
    try:
        # Suppress detailed API request parameters
        # print(f"Making API request to create subdomain with parameters: {params}")
        subdomain_result = make_whm_request("cpanel", params, method="POST")
        
        # Check if subdomain creation was successful (don't print full JSON response)
        if not subdomain_result or 'cpanelresult' not in subdomain_result or not subdomain_result['cpanelresult'].get('data'):
            error_msg = "Error creating subdomain"
            print(f"Failed to create subdomain: {error_msg}")
            
            # Try alternative method - create directories directly
            print("Attempting alternative method: creating directories directly")
            
            # Create parent directory
            mkdir_parent_params = {
                "cpanel_jsonapi_user": "orbizen",
                "cpanel_jsonapi_apiversion": "2",
                "cpanel_jsonapi_module": "Fileman",
                "cpanel_jsonapi_func": "mkdir",
                "path": f"{base_dir}/orbizen.cloud/cli/project",
                "name": project_id
            }
            
            parent_result = make_whm_request("cpanel", mkdir_parent_params, method="POST")
            # Don't print full result
            # print(f"Parent directory creation result: {parent_result}")
        else:
            print(f"Subdomain creation successful: {project_id}.orbizen.cloud")
    except Exception as e:
        print(f"Exception during subdomain creation: {str(e)}")
        # Continue with directory creation even if subdomain API fails
    
    # Create directories for the project
    try:
        # Create public_html directory
        dir_params = {
            "cpanel_jsonapi_user": "orbizen",
            "cpanel_jsonapi_apiversion": "2",
            "cpanel_jsonapi_module": "Fileman",
            "cpanel_jsonapi_func": "mkdir",
            "path": project_dir,
            "name": "public_html"
        }
        
        public_html_result = make_whm_request("cpanel", dir_params, method="POST")
        # Don't print full result
        # print(f"public_html directory creation result: {public_html_result}")
        
        # Create the phpmyadmin directory if it doesn't exist yet
        phpmyadmin_params = {
            "cpanel_jsonapi_user": "orbizen",
            "cpanel_jsonapi_apiversion": "2",
            "cpanel_jsonapi_module": "Fileman",
            "cpanel_jsonapi_func": "mkdir",
            "path": project_dir,
            "name": "phpmyadmin"
        }
        
        phpmyadmin_result = make_whm_request("cpanel", phpmyadmin_params, method="POST")
        # Don't print full result
        # print(f"phpmyadmin directory creation result: {phpmyadmin_result}")
    except Exception as e:
        print(f"Exception during directory creation: {str(e)}")
    
    # Attempt to create .htaccess file for domain routing
    try:
        # Create .htaccess in main orbizen.cloud directory
        htaccess_content = f'''
# Subdomain routing for {project_id}.orbizen.cloud
<IfModule mod_rewrite.c>
    RewriteEngine On
    RewriteCond %{{HTTP_HOST}} ^{project_id}[.]orbizen[.]cloud$ [NC]
    RewriteCond %{{REQUEST_URI}} !^/subdomains/{project_id}/
    RewriteRule ^(.*)$ /subdomains/{project_id}/public_html/$1 [L]
</IfModule>
'''
        
        # Write to a temporary file and then upload it
        temp_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), "temp_htaccess.txt")
        with open(temp_file, "w") as f:
            f.write(htaccess_content)
        
        # Upload the file using cPanel API
        with open(temp_file, "rb") as f:
            file_content = f.read()
            
        # Append to main .htaccess or create a new one
        htaccess_params = {
            "cpanel_jsonapi_user": "orbizen",
            "cpanel_jsonapi_apiversion": "2",
            "cpanel_jsonapi_module": "Fileman",
            "cpanel_jsonapi_func": "fileop",
            "op": "append",
            "file": ".htaccess",
            "dir": "/home/orbizen/public_html",
            "content": htaccess_content
        }
        
        htaccess_result = make_whm_request("cpanel", htaccess_params, method="POST")
        # Don't print full result
        # print(f"Main .htaccess update result: {htaccess_result}")
        
        # Clean up
        if os.path.exists(temp_file):
            os.remove(temp_file)
    except Exception as e:
        print(f"Exception during .htaccess creation: {str(e)}")
    
    # Create index.html file in public_html directory
    try:
        # Content for the default index.html file
        index_content = '''<!DOCTYPE html>
<html>
<head>
    <title>Welcome to Orbizen Cloud</title>
    <style>
        body { font-family: Arial, sans-serif; max-width: 800px; margin: 0 auto; padding: 20px; }
        h1 { color: #0066cc; }
        .info { background: #f5f5f5; padding: 15px; border-radius: 5px; }
    </style>
</head>
<body>
    <h1>Orbizen Cloud - Project Setup Complete</h1>
    <div class="info">
        <p>Your project has been successfully created. You can now upload your files using the Orbizen CLI:</p>
        <pre>orbizen deploy</pre>
    </div>
</body>
</html>'''
        
        # Create the index.html file
        mkfile_params = {
            "cpanel_jsonapi_user": "orbizen",
            "cpanel_jsonapi_apiversion": "2",
            "cpanel_jsonapi_module": "Fileman",
            "cpanel_jsonapi_func": "mkfile",
            "path": public_html_dir,
            "name": "index.html"
        }
        
        mkfile_result = make_whm_request("cpanel", mkfile_params, method="POST")
        # Don't print full result
        # print(f"index.html file creation result: {mkfile_result}")
        
        # Write the content to the index.html file
        write_params = {
            "cpanel_jsonapi_user": "orbizen",
            "cpanel_jsonapi_apiversion": "2",
            "cpanel_jsonapi_module": "Fileman",
            "cpanel_jsonapi_func": "fileop",
            "op": "edit",
            "file": "index.html",
            "dir": public_html_dir,
            "content": index_content
        }
        
        write_result = make_whm_request("cpanel", write_params, method="POST")
        # Don't print full result
        # print(f"index.html content writing result: {write_result}")
    except Exception as e:
        print(f"Exception during index.html creation: {str(e)}")
    
    # Ensure DNS zone is rebuilt
    try:
        dns_params = {
            "domain": "orbizen.cloud"
        }
        dns_result = make_whm_request("dnszone_reload", dns_params, method="GET")
        # Don't print full result
        # print(f"DNS zone rebuild result: {dns_result}")
    except Exception as e:
        print(f"Exception during DNS zone rebuild: {str(e)}")
    
    # Restart Apache to ensure new subdomain is active
    try:
        service_params = {
            "service": "httpd",
            "action": "restart"
        }
        apache_result = make_whm_request("configureservice", service_params, method="GET")
        # Don't print full result
        # print(f"Apache restart result: {apache_result}")
    except Exception as e:
        print(f"Exception during Apache restart: {str(e)}")
    
    # Update project_base_dir value for this project
    project_base_dir = project_dir
    
    # Try to create an Addon Domain as a backup approach
    try:
        addon_params = {
            "cpanel_jsonapi_user": "orbizen",
            "cpanel_jsonapi_apiversion": "2",
            "cpanel_jsonapi_module": "AddonDomain",
            "cpanel_jsonapi_func": "addaddondomain",
            "newdomain": f"{project_id}.orbizen.cloud",
            "subdomain": project_id,
            "dir": project_dir,
        }
        
        addon_result = make_whm_request("cpanel", addon_params, method="POST")
        # Don't print full result
        # print(f"Addon domain creation result: {addon_result}")
    except Exception as e:
        print(f"Exception during addon domain creation: {str(e)}")
    
    return {
        "subdomain": f"{project_id}.orbizen.cloud",
        "project_path": project_base_dir,
        "public_html_path": public_html_dir,
        "phpmyadmin_path": f"{project_base_dir}/phpmyadmin"
    }

def setup_custom_domain(domain, project_id):
    """Set up a custom domain for a project using cPanel API."""
    print(f"Setting up custom domain: {domain} for project {project_id}")
    
    # Create proper path for the domain - use the same path structure as subdomains
    base_dir = "/home/orbizen"
    project_dir = f"{base_dir}/orbizen.cloud/cli/project/{project_id}"
    public_html_dir = f"{project_dir}/public_html"
    
    # Create a safe subdomain name from the custom domain (replace dots with hyphens)
    safe_subdomain = domain.replace(".", "-")
    
    try:
        # First, try using the Addon Domain module which allows setting document root directly
        addon_params = {
            "cpanel_jsonapi_user": "orbizen",  
            "cpanel_jsonapi_apiversion": "2",
            "cpanel_jsonapi_module": "AddonDomain",
            "cpanel_jsonapi_func": "addaddondomain",
            "newdomain": domain,
            "subdomain": safe_subdomain,
            "dir": project_dir,
            "document_root": project_dir
        }
        
        try:
            addon_result = make_whm_request("cpanel", addon_params, method="POST")
            print(f"Addon domain setup with custom document root: {domain}")
        except Exception as e:
            print(f"Addon domain error: {str(e)}")
            
            # Fallback to using the Park module
            try:
                domain_params = {
                    "cpanel_jsonapi_user": "orbizen",  
                    "cpanel_jsonapi_apiversion": "2",
                    "cpanel_jsonapi_module": "Park",
                    "cpanel_jsonapi_func": "park",
                    "domain": domain,
                    "dir": project_dir,
                }
                
                domain_result = make_whm_request("cpanel", domain_params, method="POST")
                print(f"Domain parking result: {domain}")
            except Exception as e:
                print(f"Park domain error: {str(e)}")
        
        # Also create a subdomain as a backup approach
        try:
            # Create a subdomain with the safe name
            subdomain_params = {
                "cpanel_jsonapi_user": "orbizen",
                "cpanel_jsonapi_apiversion": "2",
                "cpanel_jsonapi_module": "SubDomain",
                "cpanel_jsonapi_func": "addsubdomain",
                "subdomain": safe_subdomain,
                "domain": "orbizen.cloud",
                "rootdomain": "orbizen.cloud",
                "dir": project_dir,
                "disallowdot": 0
            }
            
            subdomain_result = make_whm_request("cpanel", subdomain_params, method="POST")
            print(f"Created subdomain: {safe_subdomain}.orbizen.cloud for custom domain: {domain}")
        except Exception as e:
            print(f"Subdomain creation error (non-critical): {str(e)}")
            
        # Try to modify the domain's document root using the DomainLookup module
        try:
            lookup_params = {
                "cpanel_jsonapi_user": "orbizen",
                "cpanel_jsonapi_apiversion": "2",
                "cpanel_jsonapi_module": "DomainLookup",
                "cpanel_jsonapi_func": "getdocroot",
                "domain": domain
            }
            
            lookup_result = make_whm_request("cpanel", lookup_params, method="GET")
            print(f"Current document root for {domain}: {lookup_result}")
            
            # Now try to modify the document root using the correct module
            modify_params = {
                "cpanel_jsonapi_user": "orbizen",
                "cpanel_jsonapi_apiversion": "2",
                "cpanel_jsonapi_module": "Park",
                "cpanel_jsonapi_func": "park",
                "domain": domain,
                "dir": project_dir
            }
            
            modify_result = make_whm_request("cpanel", modify_params, method="POST")
            print(f"Modified document root for {domain} to {project_dir}")
        except Exception as e:
            print(f"Document root modification error (non-critical): {str(e)}")
        
        # Rebuild DNS zone for both domains
        dns_params = {
            "domain": "orbizen.cloud"
        }
        make_whm_request("dnszone_reload", dns_params, method="GET")
        
        try:
            custom_dns_params = {
                "domain": domain
            }
            make_whm_request("dnszone_reload", custom_dns_params, method="GET")
        except Exception:
            # This might fail if the domain isn't in the DNS system yet
            pass
        
        # Create .htaccess for the custom domain - similar to subdomain .htaccess
        htaccess_content = f'''
# Custom domain configuration for {domain}
<IfModule mod_rewrite.c>
    RewriteEngine On
    RewriteRule ^$ public_html/ [L]
    RewriteCond %{{REQUEST_FILENAME}} !-f
    RewriteCond %{{REQUEST_FILENAME}} !-d
    RewriteRule ^(.*)$ public_html/$1 [L]
</IfModule>

# Prevent directory listing
Options -Indexes

# PHP settings
<IfModule mod_php.c>
    php_flag display_errors Off
    php_value upload_max_filesize 64M
    php_value post_max_size 64M
    php_value max_execution_time 300
    php_value max_input_time 300
</IfModule>

# Security headers
<IfModule mod_headers.c>
    Header set X-Content-Type-Options "nosniff"
    Header set X-XSS-Protection "1; mode=block"
    Header set X-Frame-Options "SAMEORIGIN"
</IfModule>

# Prevent access to sensitive files
<FilesMatch "^([.]htaccess|[.]htpasswd|[.]git|[.]env|[.]config)">
    Order Allow,Deny
    Deny from all
</FilesMatch>
'''
        
        # Create .htaccess file
        htaccess_params = {
            "cpanel_jsonapi_user": "orbizen",
            "cpanel_jsonapi_apiversion": "2",
            "cpanel_jsonapi_module": "Fileman",
            "cpanel_jsonapi_func": "mkfile",
            "path": project_dir,
            "name": ".htaccess"
        }
        
        htaccess_result = make_whm_request("cpanel", htaccess_params, method="POST")
        
        # Write the content to the .htaccess file
        write_params = {
            "cpanel_jsonapi_user": "orbizen",
            "cpanel_jsonapi_apiversion": "2",
            "cpanel_jsonapi_module": "Fileman",
            "cpanel_jsonapi_func": "fileop",
            "op": "edit",
            "file": ".htaccess",
            "dir": project_dir,
            "content": htaccess_content
        }
        
        write_result = make_whm_request("cpanel", write_params, method="POST")
        
        # Create a symbolic link for the custom domain
        try:
            symlink_params = {
                "cpanel_jsonapi_user": "orbizen",
                "cpanel_jsonapi_apiversion": "2",
                "cpanel_jsonapi_module": "Fileman",
                "cpanel_jsonapi_func": "fileop",
                "op": "link",
                "sourcefiles": project_dir,
                "destfiles": f"/home/orbizen/public_html/{domain}",
                "doubledecode": 1
            }
            
            symlink_result = make_whm_request("cpanel", symlink_params, method="POST")
            print(f"Created symbolic link for domain {domain}")
        except Exception as e:
            print(f"Symbolic link error (non-critical): {str(e)}")
        
        # Try to update domain's document root using the Domain module
        try:
            domain_update_params = {
                "cpanel_jsonapi_user": "orbizen",
                "cpanel_jsonapi_apiversion": "2",
                "cpanel_jsonapi_module": "AddonDomain",
                "cpanel_jsonapi_func": "changedomaindir",
                "domain": domain,
                "dir": project_dir
            }
            
            domain_update_result = make_whm_request("cpanel", domain_update_params, method="POST")
            print(f"Updated document root for domain {domain} to {project_dir}")
        except Exception as e:
            print(f"Document root update error (non-critical): {str(e)}")
        
        print(f"Custom domain {domain} set up successfully. Remember to set the A record to point to the server IP.")
        return {
            "status": "success", 
            "domain": domain, 
            "project_id": project_id,
            "subdomain": f"{safe_subdomain}.orbizen.cloud",
            "document_root": project_dir,
            "server_ip": "144.76.203.197"  # Hardcoded server IP
        }
    except Exception as e:
        print(f"Error setting up custom domain {domain}: {str(e)}")
        return {
            "status": "error",
            "message": str(e)
        }

def delete_project(project_id):
    """Delete a project from Orbizen Cloud."""
    try:
        # Get project information first to find associated domains
        project_info = None
        try:
            project_response = make_api_request(f"projects/{project_id}", method="GET")
            if project_response and project_response.get('status') == 'success' and project_response.get('data'):
                project_info = project_response.get('data')
                print(f"Found project information for deletion: {project_id}")
        except Exception as e:
            print(f"Could not retrieve project information: {str(e)}")
        
        # Delete the subdomain
        try:
            # Remove the subdomain
            subdomain = f"{project_id}.orbizen.cloud"
            print(f"Removing subdomain: {subdomain}")
            
            # Use cPanel API to remove the subdomain
            subdomain_params = {
                "cpanel_jsonapi_user": "orbizen",
                "cpanel_jsonapi_apiversion": "2",
                "cpanel_jsonapi_module": "SubDomain",
                "cpanel_jsonapi_func": "delsubdomain",
                "domain": f"{project_id}.orbizen.cloud",
            }
            
            subdomain_result = make_whm_request("cpanel", subdomain_params, method="POST")
            print(f"Subdomain removal result: {subdomain}")
        except Exception as e:
            print(f"Error removing subdomain: {str(e)}")
        
        # Delete any custom domain
        if project_info and project_info.get('domain') and not project_info.get('domain').endswith('.orbizen.cloud'):
            try:
                custom_domain = project_info.get('domain')
                print(f"Removing custom domain: {custom_domain}")
                
                # Remove the domain using both Park and AddonDomain modules
                try:
                    # Try Park module first
                    park_params = {
                        "cpanel_jsonapi_user": "orbizen",
                        "cpanel_jsonapi_apiversion": "2",
                        "cpanel_jsonapi_module": "Park",
                        "cpanel_jsonapi_func": "unpark",
                        "domain": custom_domain,
                    }
                    
                    park_result = make_whm_request("cpanel", park_params, method="POST")
                    print(f"Domain unparking result: {custom_domain}")
                except Exception as e:
                    print(f"Error unparking domain: {str(e)}")
                
                try:
                    # Also try AddonDomain module
                    addon_params = {
                        "cpanel_jsonapi_user": "orbizen",
                        "cpanel_jsonapi_apiversion": "2",
                        "cpanel_jsonapi_module": "AddonDomain",
                        "cpanel_jsonapi_func": "deladdondomain",
                        "domain": custom_domain,
                    }
                    
                    addon_result = make_whm_request("cpanel", addon_params, method="POST")
                    print(f"Addon domain removal result: {custom_domain}")
                except Exception as e:
                    print(f"Error removing addon domain: {str(e)}")
                
                # Also remove the custom domain subdomain if it exists
                try:
                    # Create a safe subdomain name from the custom domain (replace dots with hyphens)
                    safe_subdomain = custom_domain.replace(".", "-")
                    
                    # Remove the subdomain
                    custom_subdomain_params = {
                        "cpanel_jsonapi_user": "orbizen",
                        "cpanel_jsonapi_apiversion": "2",
                        "cpanel_jsonapi_module": "SubDomain",
                        "cpanel_jsonapi_func": "delsubdomain",
                        "domain": f"{safe_subdomain}.orbizen.cloud",
                    }
                    
                    custom_subdomain_result = make_whm_request("cpanel", custom_subdomain_params, method="POST")
                    print(f"Custom domain subdomain removal result: {safe_subdomain}.orbizen.cloud")
                except Exception as e:
                    print(f"Error removing custom domain subdomain: {str(e)}")
                
                # Remove any symbolic links
                try:
                    symlink_params = {
                        "cpanel_jsonapi_user": "orbizen",
                        "cpanel_jsonapi_apiversion": "2",
                        "cpanel_jsonapi_module": "Fileman",
                        "cpanel_jsonapi_func": "fileop",
                        "op": "unlink",
                        "sourcefiles": f"/home/orbizen/public_html/{custom_domain}",
                        "doubledecode": 1
                    }
                    
                    symlink_result = make_whm_request("cpanel", symlink_params, method="POST")
                    print(f"Symbolic link removal result: {custom_domain}")
                except Exception as e:
                    print(f"Error removing symbolic link: {str(e)}")
            except Exception as e:
                print(f"Error handling custom domain removal: {str(e)}")
        
        # Delete project directory
        try:
            # Use cPanel API to remove the project directory
            dir_params = {
                "cpanel_jsonapi_user": "orbizen",
                "cpanel_jsonapi_apiversion": "2",
                "cpanel_jsonapi_module": "Fileman",
                "cpanel_jsonapi_func": "fileop",
                "op": "remove",
                "sourcefiles": f"/home/orbizen/orbizen.cloud/cli/project/{project_id}",
                "doubledecode": 1
            }
            
            dir_result = make_whm_request("cpanel", dir_params, method="POST")
            print(f"Project directory removal result: {project_id}")
        except Exception as e:
            print(f"Error removing project directory: {str(e)}")
        
        # Rebuild DNS zone
        try:
            dns_params = {
                "domain": "orbizen.cloud"
            }
            dns_result = make_whm_request("dnszone_reload", dns_params, method="GET")
        except Exception as e:
            print(f"Error rebuilding DNS zone: {str(e)}")
        
        # Delete from the API database
        try:
            # First try using DELETE method
            response = make_api_request(f"projects/{project_id}", method="DELETE")
            return response
        except APIError as e:
            # If DELETE method fails, try POST with delete action
            if "405" in str(e):
                print("DELETE method not allowed, trying alternative method...")
                response = make_api_request(f"projects/{project_id}/delete", method="POST")
                return response
            else:
                raise e
    except Exception as e:
        raise APIError(f"Failed to delete project: {str(e)}")

def create_subdomain_htaccess(project_id):
    """Create a .htaccess file in the subdomain's root directory."""
    try:
        # Define paths to match deploy.php
        base_dir = "/home/orbizen"
        project_dir = f"{base_dir}/orbizen.cloud/cli/project/{project_id}"
        
        # Content for the root .htaccess file
        root_htaccess_content = '''# Redirect to public_html
<IfModule mod_rewrite.c>
    RewriteEngine On
    RewriteRule ^$ public_html/ [L]
    RewriteCond %{REQUEST_FILENAME} !-f
    RewriteCond %{REQUEST_FILENAME} !-d
    RewriteRule ^(.*)$ public_html/$1 [L]
</IfModule>

# Prevent directory listing
Options -Indexes

# PHP settings for the entire project
<IfModule mod_php.c>
    php_flag display_errors Off
    php_value upload_max_filesize 64M
    php_value post_max_size 64M
    php_value max_execution_time 300
    php_value max_input_time 300
</IfModule>

# Security headers
<IfModule mod_headers.c>
    Header set X-Content-Type-Options "nosniff"
    Header set X-XSS-Protection "1; mode=block"
    Header set X-Frame-Options "SAMEORIGIN"
</IfModule>

# Prevent access to sensitive files
<FilesMatch "^([.]htaccess|[.]htpasswd|[.]git|[.]env|[.]config)">
    Order Allow,Deny
    Deny from all
</FilesMatch>
'''
        
        # Create the root .htaccess file
        root_htaccess_params = {
            "cpanel_jsonapi_user": "orbizen",
            "cpanel_jsonapi_apiversion": "2",
            "cpanel_jsonapi_module": "Fileman",
            "cpanel_jsonapi_func": "mkfile",
            "path": project_dir,
            "name": ".htaccess"
        }
        
        root_mkfile_result = make_whm_request("cpanel", root_htaccess_params, method="POST")
        
        # Write the content to the root .htaccess file
        root_write_params = {
            "cpanel_jsonapi_user": "orbizen",
            "cpanel_jsonapi_apiversion": "2",
            "cpanel_jsonapi_module": "Fileman",
            "cpanel_jsonapi_func": "fileop",
            "op": "edit",
            "file": ".htaccess",
            "dir": project_dir,
            "content": root_htaccess_content
        }
        
        root_write_result = make_whm_request("cpanel", root_write_params, method="POST")
        
        # No longer creating a .htaccess file in public_html
        
        return True
    except Exception as e:
        print(f"Exception during .htaccess creation: {str(e)}")
        return False

def deploy_project():
    """Deploy the project to Orbizen Cloud."""
    if not is_orbizen_project():
        print(f"{Fore.RED}Not an Orbizen project. Run 'orbizen init' first.{Style.RESET_ALL}")
        return False
    
    # Get the remote configuration
    remote_config = get_remote_config()
    if not remote_config:
        print(f"{Fore.RED}No remote configuration found. Run 'orbizen remote cloud' first.{Style.RESET_ALL}")
        return False
    
    # Get the latest commit
    commits = get_commits()
    if not commits:
        print(f"{Fore.RED}No commits found. Create a commit first with 'orbizen commit'.{Style.RESET_ALL}")
        return False
    
    # Check if there's a last_deployed timestamp in the remote config
    last_deployed = remote_config.get('last_deployed', 0)
    latest_commit = commits[-1]
    
    # Get the timestamp of the latest commit
    commit_timestamp = latest_commit.get('timestamp', 0)
    
    # If the latest commit is already deployed, inform the user and exit
    if last_deployed and commit_timestamp <= last_deployed:
        print(f"{Fore.YELLOW}No new changes to deploy. The latest commit is already deployed.{Style.RESET_ALL}")
        print(f"{Fore.YELLOW}Make changes and create a new commit with 'orbizen add' and 'orbizen commit' first.{Style.RESET_ALL}")
        
        # Still show the project URL as a reminder
        print(f"\n{Fore.GREEN}Your website is available at:{Style.RESET_ALL}")
        print(f"{Fore.GREEN}  https://{remote_config['domain']}{Style.RESET_ALL}")
        
        # Also remind the user about phpMyAdmin access
        print(f"\n{Fore.GREEN}You can access phpMyAdmin at:{Style.RESET_ALL}")
        print(f"{Fore.GREEN}  https://{remote_config['phpmyadmin_url']}{Style.RESET_ALL}")
        print(f"{Fore.GREEN}  Username: {remote_config['database']['user']}{Style.RESET_ALL}")
        print(f"{Fore.GREEN}  Password: {remote_config['database']['password']}{Style.RESET_ALL}")
        
        return {"status": "no_changes"}
    
    print(f"{Fore.BLUE}Deploying commit: {latest_commit['message']}{Style.RESET_ALL}")
    print(f"{Fore.BLUE}Including {len(latest_commit['files'])} files:{Style.RESET_ALL}")
    
    # Print the first 5 files
    for i, file_info in enumerate(latest_commit['files'][:5]):
        print(f"{Fore.CYAN}  - {file_info['path']} ({file_info['size']} bytes){Style.RESET_ALL}")
    
    # If there are more than 5 files, show a summary
    if len(latest_commit['files']) > 5:
        print(f"{Fore.CYAN}  ... and {len(latest_commit['files']) - 5} more files{Style.RESET_ALL}")
    
    # Create a temporary directory for the files
    temp_dir = os.path.join(get_orbizen_dir(), "temp")
    os.makedirs(temp_dir, exist_ok=True)
    
    try:
        # Calculate total size for progress tracking
        total_size = sum(file_info.get("size", 0) for file_info in latest_commit["files"])
        processed_size = 0
        
        # Copy the files to the temporary directory with progress bar
        with tqdm(total=total_size, unit='B', unit_scale=True, desc="Preparing files") as pbar:
            for file_info in latest_commit["files"]:
                src_path = file_info["path"]
                dst_path = os.path.join(temp_dir, src_path)
                file_size = file_info.get("size", 0)
                
                # Create the directory if it doesn't exist
                os.makedirs(os.path.dirname(dst_path), exist_ok=True)
                
                # Copy the file
                try:
                    shutil.copy2(src_path, dst_path)
                    processed_size += file_size
                    pbar.update(file_size)
                except Exception as e:
                    print(f"{Fore.YELLOW}Warning: Could not copy {src_path}: {str(e)}{Style.RESET_ALL}")
        
        # Create a zip file
        zip_path = os.path.join(get_orbizen_dir(), "project.zip")
        print(f"{Fore.BLUE}Creating deployment package...{Style.RESET_ALL}")
        shutil.make_archive(os.path.splitext(zip_path)[0], 'zip', temp_dir)
        
        # Get the zip file size for upload progress
        zip_size = os.path.getsize(zip_path)
        print(f"{Fore.BLUE}Package size: {zip_size / (1024*1024):.2f} MB{Style.RESET_ALL}")
        
        # Upload the zip file
        with open(zip_path, 'rb') as f:
            # Use the correct API endpoint for deployment
            project_id = remote_config['id']
            print(f"{Fore.BLUE}Uploading to Orbizen Cloud...{Style.RESET_ALL}")
            
            # Also create or update the .htaccess file to ensure proper subdomain handling
            create_subdomain_htaccess(project_id)
            
            # Get credentials for API calls
            credentials = get_global_credentials()
            
            # Use the simplified file_upload.php endpoint instead of the project deploy endpoint
            deploy_url = f"{API_ENDPOINT}file_upload.php?project_id={project_id}&user_id={credentials.get('user_id') if credentials else ''}"
            
            # Create a custom upload progress bar
            with tqdm(total=zip_size, unit='B', unit_scale=True, desc="Uploading") as upload_pbar:
                def upload_progress(monitor):
                    upload_pbar.update(monitor.bytes_read - upload_pbar.n)
                
                try:
                    # Now attempt deployment using the direct file upload endpoint with progress monitoring
                    from requests_toolbelt import MultipartEncoder, MultipartEncoderMonitor
                    
                    encoder = MultipartEncoder(
                        fields={'file': (os.path.basename(zip_path), open(zip_path, 'rb'), 'application/zip')}
                    )
                    
                    monitor = MultipartEncoderMonitor(encoder, upload_progress)
                    
                    headers = {
                        'Authorization': f"Bearer {credentials.get('token')}",
                        'Content-Type': monitor.content_type
                    }
                    
                    response = requests.post(deploy_url, data=monitor, headers=headers)
                    
                    # Debug: print the raw response
                    print(f"{Fore.YELLOW}Response status: {response.status_code}{Style.RESET_ALL}")
                    
                    # Try to parse the response as JSON
                    try:
                        response_data = response.json()
                        if response_data.get('status') == 'success':
                            print(f"{Fore.GREEN}Deployment successful! Your project is available at:{Style.RESET_ALL}")
                            print(f"{Fore.GREEN}https://{response_data.get('data', {}).get('domain', remote_config['domain'])}{Style.RESET_ALL}")
                            
                            # Also remind the user about phpMyAdmin access
                            print(f"\n{Fore.GREEN}You can access phpMyAdmin at:{Style.RESET_ALL}")
                            print(f"{Fore.GREEN}  https://{remote_config['phpmyadmin_url']}{Style.RESET_ALL}")
                            print(f"{Fore.GREEN}  Username: {remote_config['database']['user']}{Style.RESET_ALL}")
                            print(f"{Fore.GREEN}  Password: {remote_config['database']['password']}{Style.RESET_ALL}")
                            
                            # Update the last_deployed timestamp in the remote config
                            remote_config['last_deployed'] = int(time.time())
                            
                            # Save the updated remote config
                            with open(os.path.join(get_orbizen_dir(), "remote.json"), "w") as f:
                                json.dump(remote_config, f)
                        else:
                            error_msg = response_data.get('message', 'Unknown error')
                            print(f"{Fore.RED}Deployment failed: {error_msg}{Style.RESET_ALL}")
                    except ValueError:
                        # If not valid JSON, use the text directly
                        if response.status_code == 200:
                            print(f"{Fore.GREEN}Project deployed successfully!{Style.RESET_ALL}")
                            print(f"{Fore.GREEN}Your website is now available at:{Style.RESET_ALL}")
                            print(f"{Fore.GREEN}  https://{remote_config['domain']}{Style.RESET_ALL}")
                            
                            # Also remind the user about phpMyAdmin access
                            print(f"\n{Fore.GREEN}You can access phpMyAdmin at:{Style.RESET_ALL}")
                            print(f"{Fore.GREEN}  https://{remote_config['phpmyadmin_url']}{Style.RESET_ALL}")
                            print(f"{Fore.GREEN}  Username: {remote_config['database']['user']}{Style.RESET_ALL}")
                            print(f"{Fore.GREEN}  Password: {remote_config['database']['password']}{Style.RESET_ALL}")
                            
                            # Update the last_deployed timestamp in the remote config
                            remote_config['last_deployed'] = int(time.time())
                            
                            # Save the updated remote config
                            with open(os.path.join(get_orbizen_dir(), "remote.json"), "w") as f:
                                json.dump(remote_config, f)
                        else:
                            print(f"{Fore.RED}Deployment failed. Server response: {response.text[:100]}{Style.RESET_ALL}")
                except ImportError:
                    print(f"{Fore.YELLOW}Upload progress monitoring not available. Install requests-toolbelt for this feature.{Style.RESET_ALL}")
                    # Fall back to regular upload without progress monitoring
                    try:
                        response = requests.post(
                            deploy_url, 
                            files={'file': (os.path.basename(zip_path), open(zip_path, 'rb'))},
                            headers={'Authorization': f"Bearer {credentials.get('token')}"}
                        )
                        
                        # Process response as before...
                        if response.status_code == 200:
                            print(f"{Fore.GREEN}Project deployed successfully!{Style.RESET_ALL}")
                            print(f"{Fore.GREEN}Your website is now available at:{Style.RESET_ALL}")
                            print(f"{Fore.GREEN}  https://{remote_config['domain']}{Style.RESET_ALL}")
                            
                            # Also remind the user about phpMyAdmin access
                            print(f"\n{Fore.GREEN}You can access phpMyAdmin at:{Style.RESET_ALL}")
                            print(f"{Fore.GREEN}  https://{remote_config['phpmyadmin_url']}{Style.RESET_ALL}")
                            print(f"{Fore.GREEN}  Username: {remote_config['database']['user']}{Style.RESET_ALL}")
                            print(f"{Fore.GREEN}  Password: {remote_config['database']['password']}{Style.RESET_ALL}")
                            
                            # Update the last_deployed timestamp in the remote config
                            remote_config['last_deployed'] = int(time.time())
                            
                            # Save the updated remote config
                            with open(os.path.join(get_orbizen_dir(), "remote.json"), "w") as f:
                                json.dump(remote_config, f)
                    except Exception as e:
                        print(f"{Fore.RED}Error during API request: {str(e)}{Style.RESET_ALL}")
            
        # Clean up
        shutil.rmtree(temp_dir)
        os.remove(zip_path)
        
        return {"status": "success"}
    except Exception as e:
        print(f"{Fore.RED}Error deploying project: {str(e)}{Style.RESET_ALL}")
        return False 

def update_htaccess_for_visibility(project_id, visibility, allowed_ips=None):
    """Update .htaccess file to enforce visibility restrictions."""
    try:
        # Define paths
        base_dir = "/home/orbizen"
        project_dir = f"{base_dir}/orbizen.cloud/cli/project/{project_id}"
        
        # Get the base .htaccess content
        base_htaccess_content = '''# Redirect to public_html
<IfModule mod_rewrite.c>
    RewriteEngine On
    RewriteRule ^$ public_html/ [L]
    RewriteCond %{REQUEST_FILENAME} !-f
    RewriteCond %{REQUEST_FILENAME} !-d
    RewriteRule ^(.*)$ public_html/$1 [L]
</IfModule>

# Prevent directory listing
Options -Indexes

# PHP settings for the entire project
<IfModule mod_php.c>
    php_flag display_errors Off
    php_value upload_max_filesize 64M
    php_value post_max_size 64M
    php_value max_execution_time 300
    php_value max_input_time 300
</IfModule>

# Security headers
<IfModule mod_headers.c>
    Header set X-Content-Type-Options "nosniff"
    Header set X-XSS-Protection "1; mode=block"
    Header set X-Frame-Options "SAMEORIGIN"
</IfModule>

# Prevent access to sensitive files
<FilesMatch "^([.]htaccess|[.]htpasswd|[.]git|[.]env|[.]config)">
    Order Allow,Deny
    Deny from all
</FilesMatch>
'''
        
        # Add IP restrictions based on visibility
        if visibility == 'private' and allowed_ips:
            # For private visibility, only allow the specified IP
            ip_restriction = "# IP restrictions - Private access\n"
            ip_restriction += "<IfModule mod_authz_core.c>\n"
            ip_restriction += "    <RequireAll>\n"
            ip_restriction += "        Require all granted\n"
            
            # Add each IP to the allowed list
            for ip in allowed_ips:
                ip_restriction += f"        Require ip {ip}\n"
            
            ip_restriction += "    </RequireAll>\n"
            ip_restriction += "</IfModule>\n\n"
            
            # Legacy version for older Apache
            ip_restriction += "<IfModule !mod_authz_core.c>\n"
            ip_restriction += "    Order deny,allow\n"
            ip_restriction += "    Deny from all\n"
            
            # Add each IP to the allowed list
            for ip in allowed_ips:
                ip_restriction += f"    Allow from {ip}\n"
                
            ip_restriction += "</IfModule>\n"
            
            # Add the IP restriction to the htaccess content
            base_htaccess_content = ip_restriction + base_htaccess_content
            
        elif visibility == 'group' and allowed_ips:
            # For group visibility, allow the specified IPs
            ip_restriction = "# IP restrictions - Group access\n"
            ip_restriction += "<IfModule mod_authz_core.c>\n"
            ip_restriction += "    <RequireAll>\n"
            ip_restriction += "        Require all granted\n"
            
            # Add each IP to the allowed list
            for ip in allowed_ips:
                ip_restriction += f"        Require ip {ip}\n"
            
            ip_restriction += "    </RequireAll>\n"
            ip_restriction += "</IfModule>\n\n"
            
            # Legacy version for older Apache
            ip_restriction += "<IfModule !mod_authz_core.c>\n"
            ip_restriction += "    Order deny,allow\n"
            ip_restriction += "    Deny from all\n"
            
            # Add each IP to the allowed list
            for ip in allowed_ips:
                ip_restriction += f"    Allow from {ip}\n"
                
            ip_restriction += "</IfModule>\n"
            
            # Add the IP restriction to the htaccess content
            base_htaccess_content = ip_restriction + base_htaccess_content
            
        # For public visibility, no IP restrictions needed
        
        # Create or update the root .htaccess file
        root_htaccess_params = {
            "cpanel_jsonapi_user": "orbizen",
            "cpanel_jsonapi_apiversion": "2",
            "cpanel_jsonapi_module": "Fileman",
            "cpanel_jsonapi_func": "fileop",
            "op": "edit",
            "file": ".htaccess",
            "dir": project_dir,
            "content": base_htaccess_content
        }
        
        root_write_result = make_whm_request("cpanel", root_htaccess_params, method="POST")
        
        # Also create or update .htaccess in public_html to ensure restrictions apply there too
        public_html_dir = f"{project_dir}/public_html"
        
        # For public_html, we need a simpler .htaccess that inherits the main restrictions
        public_html_htaccess_content = '''# Inherit IP restrictions from parent directory
# PHP settings for public_html
<IfModule mod_php.c>
    php_flag display_errors Off
    php_value upload_max_filesize 64M
    php_value post_max_size 64M
    php_value max_execution_time 300
    php_value max_input_time 300
</IfModule>

# Prevent directory listing
Options -Indexes

# Security headers
<IfModule mod_headers.c>
    Header set X-Content-Type-Options "nosniff"
    Header set X-XSS-Protection "1; mode=block"
    Header set X-Frame-Options "SAMEORIGIN"
</IfModule>

# Prevent access to sensitive files
<FilesMatch "^([.]htaccess|[.]htpasswd|[.]git|[.]env|[.]config)">
    Order Allow,Deny
    Deny from all
</FilesMatch>
'''
        
        # Check if public_html directory exists
        check_dir_params = {
            "cpanel_jsonapi_user": "orbizen",
            "cpanel_jsonapi_apiversion": "2",
            "cpanel_jsonapi_module": "Fileman",
            "cpanel_jsonapi_func": "listfiles",
            "dir": project_dir
        }
        
        dir_result = make_whm_request("cpanel", check_dir_params, method="POST")
        
        # If public_html exists, create/update .htaccess there
        if dir_result and "data" in dir_result and any(item.get("name") == "public_html" for item in dir_result["data"]):
            public_htaccess_params = {
                "cpanel_jsonapi_user": "orbizen",
                "cpanel_jsonapi_apiversion": "2",
                "cpanel_jsonapi_module": "Fileman",
                "cpanel_jsonapi_func": "fileop",
                "op": "edit",
                "file": ".htaccess",
                "dir": public_html_dir,
                "content": public_html_htaccess_content
            }
            
            public_write_result = make_whm_request("cpanel", public_htaccess_params, method="POST")
        
        print(f"Updated .htaccess files for {visibility} visibility")
        return True
    except Exception as e:
        print(f"Exception during .htaccess update: {str(e)}")
        return False 