"""
Configuration utilities for Orbizen CLI.
"""
import os
import json
import yaml
from pathlib import Path

from orbizen import __version__

# Constants
ORBIZEN_DIR = ".orbizen"
CONFIG_FILE = "config.yaml"
CREDENTIALS_FILE = "credentials.json"
STAGING_FILE = "staged.json"
COMMITS_FILE = "commits.json"
REMOTE_FILE = "remote.json"

def get_version():
    """Return the current version of Orbizen CLI."""
    return __version__

def get_orbizen_dir():
    """Get the Orbizen directory path."""
    home = Path.home()
    orbizen_dir = home / ".orbizen"
    orbizen_dir.mkdir(exist_ok=True)
    return orbizen_dir

def get_project_orbizen_dir():
    """Get the Orbizen directory path in the current working directory."""
    cwd = Path.cwd()
    orbizen_dir = cwd / ".orbizen"
    return orbizen_dir

def get_config_path():
    """Get the path to the config file."""
    if is_orbizen_project():
        return get_project_orbizen_dir() / CONFIG_FILE
    return get_orbizen_dir() / CONFIG_FILE

def get_credentials_path():
    """Get the path to the credentials file."""
    return os.path.join(get_orbizen_dir(), CREDENTIALS_FILE)

def get_auth_path():
    """Get the auth token file path."""
    return get_orbizen_dir() / "auth.json"

def get_staging_path():
    """Get the staging file path."""
    if is_orbizen_project():
        return get_project_orbizen_dir() / "staging.json"
    return get_orbizen_dir() / "staging.json"

def get_commits_path():
    """Get the commits file path."""
    if is_orbizen_project():
        return get_project_orbizen_dir() / "commits.json"
    return get_orbizen_dir() / "commits.json"

def get_remote_path():
    """Get the remote configuration file path."""
    if is_orbizen_project():
        return get_project_orbizen_dir() / "remote.json"
    return get_orbizen_dir() / "remote.json"

def is_orbizen_project():
    """Check if the current directory is an Orbizen project."""
    return (Path.cwd() / ".orbizen").exists()

def get_config():
    """Get the configuration for the current project."""
    if not is_orbizen_project():
        return None
    
    config_path = get_config_path()
    if not os.path.isfile(config_path):
        return {}
    
    with open(config_path, 'r') as f:
        return yaml.safe_load(f)

def save_config(config):
    """Save the configuration to the config file."""
    if not is_orbizen_project():
        return False
    
    config_path = get_config_path()
    with open(config_path, 'w') as f:
        yaml.dump(config, f, default_flow_style=False)
    
    return True

def get_credentials():
    """Get the user's credentials."""
    credentials_path = get_credentials_path()
    if not os.path.isfile(credentials_path):
        return None
    
    with open(credentials_path, 'r') as f:
        return json.load(f)

def save_credentials(credentials):
    """Save the user's credentials."""
    if not is_orbizen_project():
        return False
    
    credentials_path = get_credentials_path()
    with open(credentials_path, 'w') as f:
        json.dump(credentials, f)
    
    return True

def get_user_config_dir():
    """Get the Orbizen config directory in the user's home directory."""
    home_dir = Path.home()
    config_dir = os.path.join(home_dir, ".orbizen")
    
    # Create the directory if it doesn't exist
    os.makedirs(config_dir, exist_ok=True)
    
    return config_dir

def get_global_credentials_path():
    """Get the path to the global credentials file."""
    return os.path.join(get_user_config_dir(), "credentials.json")

def get_global_credentials():
    """Get the global user credentials."""
    credentials_path = get_global_credentials_path()
    if not os.path.isfile(credentials_path):
        return None
    
    with open(credentials_path, 'r') as f:
        return json.load(f)

def save_global_credentials(credentials):
    """Save the global user credentials."""
    credentials_path = get_global_credentials_path()
    with open(credentials_path, 'w') as f:
        json.dump(credentials, f)
    
    return True

def is_authenticated():
    """Check if the user is authenticated."""
    return get_global_credentials() is not None

def save_auth_token(token):
    """Save the auth token."""
    auth_path = get_auth_path()
    with open(auth_path, 'w') as f:
        json.dump({"token": token}, f)

def get_auth_token():
    """Get the auth token."""
    auth_path = get_auth_path()
    if not auth_path.exists():
        return None
    
    try:
        with open(auth_path, 'r') as f:
            data = json.load(f)
            return data.get("token")
    except:
        return None 