"""
Credentials utilities for Orbizen CLI.
"""
import os
import json
import time
import requests
from colorama import Fore, Style
from pathlib import Path

def get_user_config_dir():
    """Get the Orbizen config directory in the user's home directory."""
    home_dir = Path.home()
    config_dir = os.path.join(home_dir, ".orbizen")
    
    # Create the directory if it doesn't exist
    os.makedirs(config_dir, exist_ok=True)
    
    return config_dir

def get_global_credentials_path():
    """Get the path to the global credentials file."""
    return os.path.join(get_user_config_dir(), "credentials.json")

def get_global_credentials():
    """Get the global user credentials."""
    credentials_path = get_global_credentials_path()
    if not os.path.isfile(credentials_path):
        return None
    
    try:
        with open(credentials_path, 'r') as f:
            return json.load(f)
    except Exception:
        return None

def save_global_credentials(credentials):
    """Save the global user credentials."""
    credentials_path = get_global_credentials_path()
    
    # Ensure directory exists
    os.makedirs(os.path.dirname(credentials_path), exist_ok=True)
    
    with open(credentials_path, 'w') as f:
        json.dump(credentials, f)
    
    return True

def clear_credentials():
    """Clear the user's credentials."""
    credentials_path = get_global_credentials_path()
    if os.path.isfile(credentials_path):
        os.remove(credentials_path)
    return True

def is_authenticated():
    """Check if the user is authenticated."""
    return get_global_credentials() is not None

def get_projects_for_user():
    """Get projects for the current user."""
    credentials = get_global_credentials()
    if not credentials or not credentials.get('user_id'):
        return []
    
    # This would typically make an API call, but for now we'll just return
    # projects associated with the user_id in the credentials
    user_id = credentials.get('user_id')
    
    # In a future implementation, this would query the server for projects
    # belonging to this user
    try:
        from orbizen.utils.api import make_api_request
        response = make_api_request(f"projects?user_id={user_id}", method="GET")
        if isinstance(response, dict) and 'data' in response:
            return response['data']
        return []
    except Exception:
        # If API call fails, return empty list
        return []

def get_credentials_file():
    """Get the path to the credentials file."""
    home = Path.home()
    creds_dir = home / ".orbizen"
    creds_dir.mkdir(exist_ok=True)
    return creds_dir / "credentials.json"

def load_credentials():
    """Load the credentials from the file."""
    creds_file = get_credentials_file()
    if creds_file.exists():
        with open(creds_file, "r") as f:
            return json.load(f)
    return None

def save_credentials(credentials):
    """Save the credentials to the file."""
    creds_file = get_credentials_file()
    with open(creds_file, "w") as f:
        json.dump(credentials, f)

def get_token():
    """Get the authentication token."""
    creds = load_credentials()
    if creds and "token" in creds:
        # Check if token is expired
        if is_token_expired(creds):
            print(f"{Fore.YELLOW}Your authentication token has expired. Trying to refresh...{Style.RESET_ALL}")
            if refresh_token(creds):
                creds = load_credentials()  # Reload after refresh
            else:
                print(f"{Fore.RED}Failed to refresh token. Please login again.{Style.RESET_ALL}")
                return None
        return creds["token"]
    return None

def is_token_expired(credentials):
    """Check if the token is expired."""
    if not credentials or "expires_at" not in credentials:
        return True
    
    # Check if the token expires in less than 5 minutes
    expires_at = credentials["expires_at"]
    current_time = time.time()
    
    # If expires_at is more than 5 minutes in the future, token is still valid
    return current_time > (expires_at - 300)

def refresh_token(credentials):
    """Refresh the authentication token."""
    if not credentials or "refresh_token" not in credentials:
        return False
    
    refresh_token = credentials.get("refresh_token")
    
    try:
        # Make request to refresh token
        response = requests.post(
            "https://orbizen.cloud/api/auth/refresh",
            json={"refresh_token": refresh_token},
            headers={"Content-Type": "application/json"}
        )
        
        if response.status_code == 200:
            # Parse the response
            data = response.json()
            
            if "token" in data and "user_id" in data:
                # Update credentials with new token
                credentials.update({
                    "token": data["token"],
                    "refresh_token": data.get("refresh_token", refresh_token),
                    "expires_at": time.time() + data.get("expires_in", 86400),
                    "user_id": data["user_id"]
                })
                
                # Save updated credentials
                save_credentials(credentials)
                return True
        
        return False
    except Exception as e:
        print(f"Error refreshing token: {str(e)}")
        return False 