"""
Media utilities for Orbizen CLI.
Handles video compression and optimization.
"""
import os
import subprocess
import shutil
import tempfile
from pathlib import Path
from colorama import Fore, Style

def check_ffmpeg_installed():
    """Check if FFmpeg is installed on the system."""
    try:
        subprocess.run(['ffmpeg', '-version'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        return True
    except (FileNotFoundError, subprocess.SubprocessError):
        return False

def compress_video(input_path, output_path=None, quality='medium', max_size_mb=50):
    """
    Compress a video file using FFmpeg.
    
    Args:
        input_path (str): Path to the input video file
        output_path (str, optional): Path to save the compressed video. If None, will use input_path with '_compressed' suffix
        quality (str): Compression quality - 'low', 'medium', or 'high'
        max_size_mb (int): Target maximum file size in MB
    
    Returns:
        str: Path to the compressed video file, or None if compression failed
    """
    if not check_ffmpeg_installed():
        print(f"{Fore.RED}FFmpeg is not installed. Please install FFmpeg to use video compression.{Style.RESET_ALL}")
        print(f"{Fore.YELLOW}Visit https://ffmpeg.org/download.html for installation instructions.{Style.RESET_ALL}")
        return None
    
    # If no output path is provided, create one based on the input path
    if output_path is None:
        file_name, file_ext = os.path.splitext(input_path)
        output_path = f"{file_name}_compressed{file_ext}"
    
    # Create temp directory for processing
    temp_dir = tempfile.mkdtemp()
    temp_output = os.path.join(temp_dir, os.path.basename(output_path))
    
    try:
        # Get video information
        info_cmd = ['ffprobe', '-v', 'error', '-select_streams', 'v:0', 
                   '-show_entries', 'stream=width,height,duration', 
                   '-of', 'csv=p=0', input_path]
        
        result = subprocess.run(info_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
        if result.returncode != 0:
            print(f"{Fore.RED}Error getting video information: {result.stderr}{Style.RESET_ALL}")
            return None
        
        # Parse video information
        try:
            width, height, duration = result.stdout.strip().split(',')
            width, height = int(width), int(height)
            duration = float(duration)
        except ValueError:
            print(f"{Fore.YELLOW}Warning: Could not parse video information. Using default compression settings.{Style.RESET_ALL}")
            width, height = 1920, 1080  # Default to full HD
            duration = 60  # Default to 1 minute
        
        # Calculate target bitrate based on quality and max size
        # Formula: bitrate = (target_size_bytes * 8) / duration_seconds
        target_size_bytes = max_size_mb * 1024 * 1024 * 0.95  # 95% of max size to be safe
        
        # Quality multipliers
        quality_settings = {
            'low': {'crf': 28, 'preset': 'faster', 'scale': 0.7},
            'medium': {'crf': 23, 'preset': 'medium', 'scale': 0.85},
            'high': {'crf': 20, 'preset': 'slow', 'scale': 1.0}
        }
        
        settings = quality_settings.get(quality, quality_settings['medium'])
        
        # Calculate new dimensions if needed
        new_width = int(width * settings['scale'])
        new_height = int(height * settings['scale'])
        
        # Ensure dimensions are even (required by some codecs)
        new_width = new_width - (new_width % 2)
        new_height = new_height - (new_height % 2)
        
        # Calculate target bitrate
        target_video_bitrate = int((target_size_bytes * 8) / duration * 0.85)  # 85% for video, rest for audio
        target_audio_bitrate = int((target_size_bytes * 8) / duration * 0.15)
        
        # Minimum bitrates to ensure quality
        min_video_bitrate = 500000  # 500 kbps
        min_audio_bitrate = 64000   # 64 kbps
        
        target_video_bitrate = max(target_video_bitrate, min_video_bitrate)
        target_audio_bitrate = max(target_audio_bitrate, min_audio_bitrate)
        
        print(f"{Fore.BLUE}Compressing video: {os.path.basename(input_path)}{Style.RESET_ALL}")
        print(f"{Fore.BLUE}Quality: {quality}, Target size: {max_size_mb}MB{Style.RESET_ALL}")
        print(f"{Fore.BLUE}Original: {width}x{height}, New: {new_width}x{new_height}{Style.RESET_ALL}")
        
        # Compress the video
        cmd = [
            'ffmpeg',
            '-i', input_path,
            '-c:v', 'libx264',
            '-crf', str(settings['crf']),
            '-preset', settings['preset'],
            '-c:a', 'aac',
            '-b:a', f'{int(target_audio_bitrate/1000)}k',
            '-vf', f'scale={new_width}:{new_height}',
            '-movflags', '+faststart',  # Optimize for web streaming
            '-y',  # Overwrite output file if it exists
            temp_output
        ]
        
        result = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        
        if result.returncode != 0:
            print(f"{Fore.RED}Error compressing video: {result.stderr.decode('utf-8', errors='replace')}{Style.RESET_ALL}")
            return None
        
        # Check if the compressed file is actually smaller
        original_size = os.path.getsize(input_path)
        compressed_size = os.path.getsize(temp_output)
        
        if compressed_size >= original_size:
            print(f"{Fore.YELLOW}Warning: Compressed file is not smaller than the original. Using original file.{Style.RESET_ALL}")
            return input_path
        
        # Copy the compressed file to the output path
        shutil.copy2(temp_output, output_path)
        
        # Calculate compression ratio
        compression_ratio = (1 - (compressed_size / original_size)) * 100
        print(f"{Fore.GREEN}Compression successful! Reduced by {compression_ratio:.2f}% ({original_size / (1024*1024):.2f}MB → {compressed_size / (1024*1024):.2f}MB){Style.RESET_ALL}")
        
        return output_path
    
    except Exception as e:
        print(f"{Fore.RED}Error during video compression: {str(e)}{Style.RESET_ALL}")
        return None
    
    finally:
        # Clean up temp directory
        shutil.rmtree(temp_dir, ignore_errors=True)

def batch_compress_videos(directory, quality='medium', max_size_mb=50, extensions=None):
    """
    Compress all videos in a directory.
    
    Args:
        directory (str): Directory containing videos to compress
        quality (str): Compression quality - 'low', 'medium', or 'high'
        max_size_mb (int): Target maximum file size in MB
        extensions (list): List of video file extensions to process. Defaults to common video formats.
    
    Returns:
        dict: Dictionary mapping original paths to compressed paths
    """
    if extensions is None:
        extensions = ['.mp4', '.mov', '.avi', '.mkv', '.webm', '.flv', '.wmv']
    
    compressed_files = {}
    
    for root, _, files in os.walk(directory):
        for file in files:
            if any(file.lower().endswith(ext) for ext in extensions):
                input_path = os.path.join(root, file)
                
                # Skip if file is already under the size limit
                if os.path.getsize(input_path) <= max_size_mb * 1024 * 1024:
                    print(f"{Fore.BLUE}Skipping {file} - already under {max_size_mb}MB limit{Style.RESET_ALL}")
                    continue
                
                # Create output path in the same directory with '_compressed' suffix
                file_name, file_ext = os.path.splitext(file)
                output_file = f"{file_name}_compressed{file_ext}"
                output_path = os.path.join(root, output_file)
                
                # Compress the video
                result = compress_video(input_path, output_path, quality, max_size_mb)
                
                if result:
                    compressed_files[input_path] = result
    
    return compressed_files 