"""
Project utilities for Orbizen CLI.
"""
import os
import json
import time
import shutil
import random
import string
import hashlib
from datetime import datetime
from pathlib import Path
from tqdm import tqdm
from colorama import Fore, Style

from orbizen.utils.config import (
    get_orbizen_dir,
    get_staging_path,
    get_commits_path,
    get_remote_path,
    is_orbizen_project,
)
from orbizen.utils.api import (
    create_project_directory,
    setup_phpmyadmin,
    create_database,
    create_subdomain,
    setup_custom_domain,
    create_subdomain_htaccess,
    make_api_request,
)
from orbizen.utils.credentials import get_global_credentials

def generate_project_id(length=6):
    """Generate a random project ID."""
    letters = string.ascii_lowercase + string.digits
    return ''.join(random.choice(letters) for _ in range(length))

def calculate_file_hash(file_path):
    """Calculate the SHA-256 hash of a file."""
    sha256_hash = hashlib.sha256()
    
    with open(file_path, "rb") as f:
        # Read and update hash in chunks of 4K
        for byte_block in iter(lambda: f.read(4096), b""):
            sha256_hash.update(byte_block)
    
    return sha256_hash.hexdigest()

def get_staged_files():
    """Get the staged files for the current project."""
    if not is_orbizen_project():
        return []
    
    staging_path = get_staging_path()
    if not os.path.isfile(staging_path):
        return []
    
    with open(staging_path, 'r') as f:
        return json.load(f)

def save_staged_files(files):
    """Save the staged files."""
    if not is_orbizen_project():
        return False
    
    staging_path = get_staging_path()
    with open(staging_path, 'w') as f:
        json.dump(files, f)
    
    return True

def get_file_info(file_path):
    """Get information about a file."""
    try:
        abs_path = os.path.abspath(file_path)
        rel_path = os.path.relpath(abs_path, os.getcwd())
        
        # Skip files in .orbizen directory
        if rel_path.startswith(os.path.join(".", ".orbizen")):
            return None
            
        # Skip hidden files and directories (starting with .)
        if os.path.basename(rel_path).startswith('.'):
            return None
        
        # Skip files that don't exist
        if not os.path.isfile(abs_path):
            return None
        
        # Skip compressed file formats
        compressed_extensions = ['.zip', '.tar', '.gz', '.tgz', '.bz2', '.xz', '.7z', '.rar', '.tar.gz', '.tar.bz2', '.tar.xz']
        file_ext = os.path.splitext(rel_path.lower())[1]
        full_path_lower = rel_path.lower()
        
        # Check both simple extension and combined extensions like .tar.gz
        if file_ext in compressed_extensions or any(full_path_lower.endswith(ext) for ext in compressed_extensions):
            print(f"Skipping compressed file: {rel_path}")
            return None
        
        # Skip files larger than 150 MB
        file_size = os.path.getsize(abs_path)
        if file_size > 150 * 1024 * 1024:  # 150 MB
            print(f"Skipping large file ({file_size / 1024 / 1024:.2f} MB): {rel_path}")
            return None
        
        # Calculate the hash
        file_hash = calculate_file_hash(abs_path)
        
        return {
            "path": rel_path,
            "hash": file_hash,
            "size": file_size,
            "modified": os.path.getmtime(abs_path),
        }
    except Exception as e:
        print(f"Error processing file {file_path}: {str(e)}")
        return None

def stage_file(file_path):
    """Stage a file for commit."""
    if not is_orbizen_project():
        return False
    
    # Get the staged files
    staged_files = get_staged_files()
    
    # Get info about the file
    file_info = get_file_info(file_path)
    if file_info is None:
        return False
    
    # Update the staged files
    staged_files.append(file_info)
    
    # Save the staged files
    return save_staged_files(staged_files)

def stage_directory(dir_path):
    """Stage all files in a directory."""
    if not is_orbizen_project():
        return False
    
    # Get the staged files
    staged_files = get_staged_files()
    
    # First, count the total number of files to process
    total_files = 0
    for root, _, files in os.walk(dir_path):
        total_files += len(files)
    
    print(f"Scanning {total_files} files in {dir_path}...")
    
    # Create a progress bar
    with tqdm(total=total_files, desc="Staging files", unit="file") as pbar:
        staged_count = 0
        skipped_count = 0
        
        # Walk the directory
        for root, _, files in os.walk(dir_path):
            for file in files:
                file_path = os.path.join(root, file)
                file_info = get_file_info(file_path)
                
                # Update progress bar
                pbar.update(1)
                
                if file_info is not None:
                    # Check if file is already staged (avoid duplicates)
                    is_duplicate = False
                    for existing in staged_files:
                        if existing.get('path') == file_info['path']:
                            is_duplicate = True
                            break
                    
                    if not is_duplicate:
                        staged_files.append(file_info)
                        staged_count += 1
                        pbar.set_postfix(staged=staged_count, skipped=skipped_count)
                else:
                    skipped_count += 1
                    pbar.set_postfix(staged=staged_count, skipped=skipped_count)
    
    # Save the staged files
    if staged_count > 0:
        print(f"{Fore.GREEN}Staged {staged_count} files from {dir_path}{Style.RESET_ALL}")
        return save_staged_files(staged_files)
    else:
        print(f"{Fore.YELLOW}No new files to stage from {dir_path}{Style.RESET_ALL}")
        return True

def get_commits():
    """Get the commits for the current project."""
    if not is_orbizen_project():
        return []
    
    commits_path = get_commits_path()
    if not os.path.isfile(commits_path):
        return []
    
    with open(commits_path, 'r') as f:
        return json.load(f)

def save_commits(commits):
    """Save the commits."""
    if not is_orbizen_project():
        return False
    
    commits_path = get_commits_path()
    with open(commits_path, 'w') as f:
        json.dump(commits, f)
    
    return True

def create_commit(message):
    """Create a commit."""
    if not is_orbizen_project():
        return False
    
    # Get the staged files
    staged_files = get_staged_files()
    if not staged_files:
        return False
    
    # Show progress when processing files
    print(f"Processing {len(staged_files)} files for commit...")
    
    # Process files with progress bar
    processed_files = []
    with tqdm(total=len(staged_files), desc="Creating commit", unit="file") as pbar:
        for file_info in staged_files:
            # Verify file still exists and matches hash
            file_path = file_info.get('path')
            if os.path.isfile(file_path):
                current_hash = calculate_file_hash(file_path)
                if current_hash == file_info.get('hash'):
                    processed_files.append(file_info)
                else:
                    print(f"{Fore.YELLOW}Warning: File {file_path} has changed since staging. Using latest version.{Style.RESET_ALL}")
                    # Update the hash
                    file_info['hash'] = current_hash
                    file_info['size'] = os.path.getsize(file_path)
                    file_info['modified'] = os.path.getmtime(file_path)
                    processed_files.append(file_info)
            else:
                print(f"{Fore.YELLOW}Warning: File {file_path} no longer exists. Skipping.{Style.RESET_ALL}")
            
            # Update progress
            pbar.update(1)
    
    # Get the commits
    commits = get_commits()
    
    # Create the commit
    commit = {
        "id": len(commits) + 1,
        "message": message,
        "timestamp": datetime.now().isoformat(),
        "files": processed_files,
    }
    
    # Add the commit
    commits.append(commit)
    
    # Save the commits
    result = save_commits(commits)
    
    # Clear the staged files
    if result:
        save_staged_files([])
        print(f"{Fore.GREEN}Successfully created commit: {message}{Style.RESET_ALL}")
    
    return result

def get_remote_config():
    """Get the remote configuration."""
    if not is_orbizen_project():
        return None
    
    remote_path = get_remote_path()
    if not os.path.isfile(remote_path):
        return None
    
    with open(remote_path, 'r') as f:
        return json.load(f)

def save_remote_config(config):
    """Save the remote configuration."""
    if not is_orbizen_project():
        return False
    
    remote_path = get_remote_path()
    with open(remote_path, 'w') as f:
        json.dump(config, f)
    
    return True

def create_project(name, domain=None, project_type="PHP", php_version="8.2"):
    """Create a new project on Orbizen Cloud."""
    # Get the current user ID
    credentials = get_global_credentials()
    user_id = credentials.get('user_id') if credentials else None
    
    if not user_id:
        print(f"{Fore.RED}You must be logged in to create a project. Run 'orbizen login' first.{Style.RESET_ALL}")
        return None
        
    # Generate a project ID
    project_id = f"{name.lower().replace(' ', '-')}-{generate_project_id()}"
    
    # Create the project on the server
    try:
        create_project_directory(project_id)
        
        # Create database credentials
        db_name = f"orbizen_{project_id.replace('-', '_')}"
        db_user = f"orbizen_{generate_project_id(8)}"
        db_pass = ''.join(random.choice(string.ascii_letters + string.digits) for _ in range(16))
        
        # Create the database
        create_database(db_name, db_user, db_pass)
        
        # Set up phpMyAdmin (at project root level)
        setup_phpmyadmin(project_id, db_name, db_user, db_pass)
        
        # Create the project directory structure and .htaccess files
        create_subdomain(project_id)
        
        # Create the .htaccess files for proper subdomain handling
        create_subdomain_htaccess(project_id)
        
        # Set up custom domain if provided
        if domain:
            setup_custom_domain(domain, project_id)
        
        # Create the remote configuration
        remote_config = {
            "id": project_id,
            "name": name,
            "domain": domain if domain else f"{project_id}.orbizen.cloud",
            "type": project_type,
            "php_version": php_version,
            "database": {
                "name": db_name,
                "user": db_user,
                "password": db_pass,
            },
            "created_at": datetime.now().isoformat(),
            "user_id": user_id,
            "phpmyadmin_url": f"{project_id}.orbizen.cloud/phpmyadmin"
        }
        
        # Save the remote configuration
        save_remote_config(remote_config)
        
        # Register the project in the API database
        registered = False
        max_retries = 3
        retry_count = 0
        
        while not registered and retry_count < max_retries:
            try:
                project_data = {
                    "user_id": user_id,
                    "project_id": project_id,
                    "name": name,
                    "domain": domain if domain else f"{project_id}.orbizen.cloud",
                    "type": project_type,
                    "php_version": php_version,
                    "visibility": "private"  # Default to private visibility
                }
                api_response = make_api_request("projects", method="POST", data=project_data)
                
                if api_response and api_response.get('status') == 'success':
                    registered = True
                    print(f"{Fore.GREEN}Project registered successfully in database.{Style.RESET_ALL}")
                else:
                    error_msg = api_response.get('message', 'Unknown error') if api_response else 'No response'
                    print(f"{Fore.YELLOW}Warning: API registration returned error: {error_msg}{Style.RESET_ALL}")
                    retry_count += 1
                    time.sleep(1)  # Wait 1 second before retrying
            except Exception as e:
                print(f"{Fore.YELLOW}Warning: Failed to register project in database: {str(e)}{Style.RESET_ALL}")
                retry_count += 1
                time.sleep(1)  # Wait 1 second before retrying
        
        if not registered:
            print(f"{Fore.YELLOW}Warning: Project files created locally but registration in cloud database failed.{Style.RESET_ALL}")
            print(f"{Fore.YELLOW}You may need to manually register this project or try again later.{Style.RESET_ALL}")
        
        return remote_config
    except Exception as e:
        print(f"{Fore.RED}Error creating project: {str(e)}{Style.RESET_ALL}")
        return None

def deploy_project():
    """Deploy the project to Orbizen Cloud."""
    if not is_orbizen_project():
        return False
    
    # Get the remote configuration
    remote_config = get_remote_config()
    if not remote_config:
        print(f"{Fore.RED}No remote configuration found. Run 'orbizen remote cloud' first.{Style.RESET_ALL}")
        return False
    
    # Get the latest commit
    commits = get_commits()
    if not commits:
        print(f"{Fore.RED}No commits found. Create a commit first with 'orbizen commit'.{Style.RESET_ALL}")
        return False
    
    latest_commit = commits[-1]
    print(f"{Fore.BLUE}Deploying commit: {latest_commit['message']}{Style.RESET_ALL}")
    print(f"{Fore.BLUE}Including {len(latest_commit['files'])} files:{Style.RESET_ALL}")
    
    # Print the first 5 files
    for i, file_info in enumerate(latest_commit['files'][:5]):
        print(f"{Fore.CYAN}  - {file_info['path']} ({file_info['size']} bytes){Style.RESET_ALL}")
    
    # If there are more than 5 files, show a summary
    if len(latest_commit['files']) > 5:
        print(f"{Fore.CYAN}  ... and {len(latest_commit['files']) - 5} more files{Style.RESET_ALL}")
    
    # Find deleted files by comparing with previous commit
    deleted_files = []
    if len(commits) > 1:
        # Get a list of all files that have ever been in the project
        all_previous_files = set()
        for commit in commits[:-1]:  # All commits except the latest one
            for file_info in commit['files']:
                if not file_info['path'].startswith('.orbizen/'):
                    all_previous_files.add(file_info['path'])
        
        # Current files in the latest commit
        current_file_paths = {file_info['path'] for file_info in latest_commit['files'] 
                             if not file_info['path'].startswith('.orbizen/')}
        
        # Find files that were in previous commits but not in the current one
        for file_path in all_previous_files:
            if file_path not in current_file_paths:
                deleted_files.append(file_path)
    
    # If any files were deleted, show them
    if deleted_files:
        print(f"{Fore.YELLOW}Files to be deleted: {len(deleted_files)}{Style.RESET_ALL}")
        for i, file_path in enumerate(deleted_files[:5]):
            print(f"{Fore.YELLOW}  - {file_path}{Style.RESET_ALL}")
        
        if len(deleted_files) > 5:
            print(f"{Fore.YELLOW}  ... and {len(deleted_files) - 5} more files{Style.RESET_ALL}")
    
    # Create a temporary directory for the files
    temp_dir = os.path.join(get_orbizen_dir(), "temp")
    os.makedirs(temp_dir, exist_ok=True)
    
    try:
        # Copy the files to the temporary directory
        for file_info in tqdm(latest_commit["files"], desc="Preparing files"):
            src_path = file_info["path"]
            dst_path = os.path.join(temp_dir, src_path)
            
            # Create the directory if it doesn't exist
            os.makedirs(os.path.dirname(dst_path), exist_ok=True)
            
            # Copy the file
            try:
                shutil.copy2(src_path, dst_path)
            except Exception as e:
                print(f"{Fore.YELLOW}Warning: Could not copy {src_path}: {str(e)}{Style.RESET_ALL}")
        
        # Create a file with the list of deleted files
        if deleted_files:
            deleted_files_path = os.path.join(temp_dir, ".orbizen_deleted_files")
            with open(deleted_files_path, 'w') as f:
                for file_path in deleted_files:
                    f.write(f"{file_path}\n")
        
        # Create a zip file
        zip_path = os.path.join(get_orbizen_dir(), "project.zip")
        print(f"{Fore.BLUE}Creating deployment package at {zip_path}{Style.RESET_ALL}")
        shutil.make_archive(os.path.splitext(zip_path)[0], 'zip', temp_dir)
        
        # Upload the zip file
        with open(zip_path, 'rb') as f:
            files = {'file': (os.path.basename(zip_path), f)}
            
            # Use the correct API endpoint for deployment
            project_id = remote_config['id']
            print(f"{Fore.BLUE}Deploying project {project_id} to Orbizen Cloud...{Style.RESET_ALL}")
            
            # Also create or update the .htaccess file to ensure proper subdomain handling
            create_subdomain_htaccess(project_id)
            
            response = make_api_request(
                f"projects/{project_id}/deploy",
                method="POST",
                files=files,
            )
            
            if response and response.get('status') == 'success':
                print(f"{Fore.GREEN}Deployment successful! Your project is available at:{Style.RESET_ALL}")
                print(f"{Fore.GREEN}https://{response.get('data', {}).get('domain', remote_config['domain'])}{Style.RESET_ALL}")
            else:
                error_msg = response.get('message', 'Unknown error') if response else 'No response'
                print(f"{Fore.RED}Deployment failed: {error_msg}{Style.RESET_ALL}")
        
        # Clean up
        shutil.rmtree(temp_dir)
        os.remove(zip_path)
        
        return response
    except Exception as e:
        print(f"{Fore.RED}Error deploying project: {str(e)}{Style.RESET_ALL}")
        return False

def get_projects():
    """Get the user's projects."""
    try:
        response = make_api_request("projects")
        if not response:
            print(f"{Fore.YELLOW}No response from API when fetching projects.{Style.RESET_ALL}")
            return {"status": "error", "message": "No response from API", "data": []}
        
        # Return the full response object so callers can check status and extract data
        return response
    except Exception as e:
        print(f"{Fore.RED}Error getting projects: {str(e)}{Style.RESET_ALL}")
        return {"status": "error", "message": str(e), "data": []} 